-- Init: sets up the structure in which we'll store our data and functions
--
local addOnName, RPTAGS = ...;

RPTAGS.addOnName = addOnName;

for _, item in ipairs({ 
    "cache", 
    "const",
    "get", 
    "locale", 
    "startup", 
    "utils", 
    }) do RPTAGS[item] = {} end;


local startupFrame = CreateFrame("frame")
startupFrame:RegisterEvent("ADDON_LOADED");
startupFrame:SetScript("OnEvent", 
  function(self, event, addOnLoaded)
    if addOnLoaded == addOnName
       then if not RP_TagsDB then RP_TagsDB = {} end;
            -- figure out which mode we're running in
            if     _G["ElvUI"] and RP_TagsDB.mode == "parallel" then RPTAGS.mode = "parallel"
            elseif _G["ElvUI"] and RP_TagsDB.mode == "elvui"    then RPTAGS.mode = "elvui"
            elseif _G["ElvUI"]                                  then RPTAGS.mode = "elvui"
            else                                                     RPTAGS.mode = "standalone"
            end; -- if

            if RPTAGS.mode ~= "standalone" then RP_TagsDB.mode = RPTAGS.mode; end;  -- only save it if we're not in standalone mode

            -- figure out what our MSP client is
            local have = {};
            -- if _G["mrp"]      then table.insert(have, "mrp")  end;
            if _G["TRP3_API"] then table.insert(have, "trp3") end;
            -- if _G["xrp"]      then table.insert(have, "xrp")  end;

            RPTAGS.cache.started = { "Description: This table contains a list of each startup routine excuted, in the order in which they were executed." };

            if     #have == 1 
            then   RPTAGS.msp = have[1]
                   for _, startupRoutine in ipairs(RPTAGS.startup) 
                   do  table.insert(RPTAGS.cache.started, startupRoutine(RPTAGS.mode));
                   end; -- for
            else   for _, startupRoutine in ipairs(RPTAGS.startup)
                   do  local started = startupRoutine(RPTAGS.mode);
                       table.insert(RPTAGS.cache.started, started);
                       if started == "utils-locale" then break end; -- we have enough to give error messages
                   end; -- for

                   StaticPopupDialogs["RPTAGS_NO_MSP_CLIENT_FOUND"] = 
                     { text = RPTAGS.utils.locale.loc("NO_MSP_CLIENT"),
                       button1 = RPTAGS.utils.locale.loc("ERROR_ACKNOWLEDGED"),
                       showAlert = true,
                       whileDead = true,
                       timeout = 0,
                   };

                   local addonsFound = {};
                   for _, addon in ipairs(have) do table.insert(addonsFound, RPTAGS.utils.locale.loc("ADDON_" .. string.upper(addon))) end;

                   StaticPopupDialogs["RPTAGS_TOO_MANY_MSP_CLIENTS"] =
                     { text = RPTAGS.utils.locale.loc("TOO_MANY_MSP") .. table.concat(addonsFound, ", "),
                       button1 = RPTAGS.utils.locale.loc("ERROR_ACKNOWLEDGED"),
                       showAlert = true,
                       whileDead = true,
                       timeout = 0,
                     };

                   if   #have == 0 
                   then StaticPopup_Show("RPTAGS_NO_MSP_CLIENT_FOUND")
                   else StaticPopup_Show("RPTAGS_TOO_MANY_MSP_CLIENTS")
                   end;

            end; -- if #have == 1

       end; -- if addOnLoaded
    end); -- function
_G.RPTAGS = RPTAGS;
