-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
--
-- The licensor cannot revoke these freedoms as long as you follow the
-- license terms.
--
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Get-Mrp: functions for getting data from mrp
  --
  -- function list at end

local RPTAGS           = RPTAGS;
  table.insert(RPTAGS.startup, function() -- startup function

  if   RPTAGS.msp == "mrp"
  then

    RPTAGS.get = { core = {}, color = {}, text = {}, icon = {}, };

    -- constants
    --
    local Const            = RPTAGS.const;
    local ICON             = Const.icons;
    local STATUS           = ICON.STATUS;
    local GENDER           = ICON.GENDER; -- category
    local PARAMS           = ICON.PARAMS;  -- category
    local BIG_PROFILE = Const.BIG_PROFILE;

    -- libraries
    --
    local LRI              = LibStub("LibRealmInfo");

    -- utilities
    --
    local loc              = RPTAGS.utils.locale.loc;
    local split            = RPTAGS.utils.text.split;
    local tc               = RPTAGS.utils.text.titlecase;
    local Config           = RPTAGS.utils.config;
    local sanitize         = RPTAGS.utils.text.sanitize;
    local redToGreenHue    = RPTAGS.utils.color.redToGreenHue;
    local compareColor     = RPTAGS.utils.color.compare;
    local hsvToRgb         = RPTAGS.utils.color.hsvToRgb;
    local colorCode        = RPTAGS.utils.color.colorCode;
    local splitRGB         = RPTAGS.utils.color.hexaToNumber;
    local match            = RPTAGS.utils.text.match;

    -- wow utilities
    local BATTLENET_FONT_COLOR       = BATTLENET_FONT_COLOR;
    local BNGetFriendGameAccountInfo = BNGetFriendGameAccountInfo;
    local BNGetNumFriendGameAccounts = BNGetNumFriendGameAccounts;
    local BNGetNumFriends            = BNGetNumFriends;
    local C_FriendList               = C_FriendList;
    local CreateFrame                = CreateFrame;
    local DIM_RED_FONT_COLOR         = DIM_RED_FONT_COLOR;
    local GREEN_FONT_COLOR           = GREEN_FONT_COLOR;
    local GetGuildInfo               = GetGuildInfo;
    local IsInGuild                  = IsInGuild;
    local IsInRaid                   = IsInRaid;
    local LIGHTBLUE_FONT_COLOR       = LIGHTBLUE_FONT_COLOR;
    local LIGHTGRAY_FONT_COLOR       = LIGHTGRAY_FONT_COLOR;
    local ORANGE_FONT_COLOR          = ORANGE_FONT_COLOR;
    local UnitAura                   = UnitAura;
    local UnitClassBase              = UnitClassBase;
    local UnitGUID                   = UnitGUID;
    local UnitInOtherParty           = UnitInOtherParty;
    local UnitInParty                = UnitInParty;
    local UnitIsInMyGuild            = UnitIsInMyGuild;
    local UnitIsPVP                  = UnitIsPVP;
    local UnitIsPlayer               = UnitIsPlayer;
    local UnitIsSameServer           = UnitIsSameServer;
    local UnitName                   = UnitName;
    local UnitRace                   = UnitRace;
    local UnitSex                    = UnitSex;

    --
    RPTAGS.get.color.age         = getRelativeAgeColor;
    RPTAGS.get.color.custom      = getColor;
    RPTAGS.get.color.eye         = getEyeColor;
    RPTAGS.get.color.friend      = getFriendColor;
    RPTAGS.get.color.gender      = getGenderColor;
    RPTAGS.get.color.guild       = getGuildColor;
    RPTAGS.get.color.guildstatus = getGuildStatusColor;
    RPTAGS.get.color.height      = getRelativeHeightColor;
    RPTAGS.get.color.hilite      = getHiliteColor;
    RPTAGS.get.color.name        = getNameColor;
    RPTAGS.get.color.party       = getPartyColor;
    RPTAGS.get.color.profileSize = computeProfileSizeColor;
    RPTAGS.get.color.relation    = getRelationColor;
    RPTAGS.get.color.status      = getStatusColor;
    RPTAGS.get.color.weight      = getRelativeWeightColor;
    RPTAGS.get.core.chara        = getChara;
    RPTAGS.get.core.data         = getData;
    RPTAGS.get.core.field        = getField;
    RPTAGS.get.core.misc         = getMisc;
    RPTAGS.get.core.profieSize   = getProfileSizeTag;
    RPTAGS.get.core.unitID       = getUnitID;
    RPTAGS.get.icon.gender       = getGenderIcon;
    RPTAGS.get.icon.glances      = getGlanceIcons;
    RPTAGS.get.icon.pvp          = pvpIcon;
    RPTAGS.get.icon.relation     = relationshipIcon;
    RPTAGS.get.icon.status       = getStatusIcon;
    RPTAGS.get.icon.unit         = unitIcon;
    RPTAGS.get.text.age          = getAge;
    RPTAGS.get.text.birthplace   = getBP;
    RPTAGS.get.text.class        = getClass;
    RPTAGS.get.text.client       = getClientInfo;
    RPTAGS.get.text.curr         = getCurr;
    RPTAGS.get.text.eyes         = getEyes;
    RPTAGS.get.text.firstname    = getFirstName;
    RPTAGS.get.text.fulltitle    = getFT;
    RPTAGS.get.text.gender       = getGender;
    RPTAGS.get.text.glance       = getGlance;
    RPTAGS.get.text.guild        = getGuildInfo;
    RPTAGS.get.text.height       = getHeight;
    RPTAGS.get.text.home         = getHome;
    RPTAGS.get.text.ic           = getIcStatus;
    RPTAGS.get.text.info         = getInfo;
    RPTAGS.get.text.knownName    = getKnownName;
    RPTAGS.get.text.lastname     = getLastName;
    RPTAGS.get.text.misc         = getMiscAndTweakOutput;
    RPTAGS.get.text.name         = getName;
    RPTAGS.get.text.note         = scanNote;
    RPTAGS.get.text.npc          = getNpcStatus;
    RPTAGS.get.text.ooc          = getOocStatus;
    RPTAGS.get.text.pronoun      = getPronoun;
    RPTAGS.get.text.pronouns     = getAllPronouns;
    RPTAGS.get.text.race         = getRace;
    RPTAGS.get.text.relation     = getRelation;
    RPTAGS.get.text.relwho       = getRelWho;
    RPTAGS.get.text.server       = getServerInfo;
    RPTAGS.get.text.sizebuff     = getSizeBuff;
    RPTAGS.get.text.status       = getStatus;
    RPTAGS.get.text.style        = getRoleplayingStyle;
    RPTAGS.get.text.title        = getTitle;
    RPTAGS.get.text.weight       = getWeight;
    RPTAGS.get.text.xp           = getExperience;
    RPTAGS.get.text.years        = getYears;
    return "get-mrp";

  else return "get-mrp-inactive";

  end; -- if mrp
end); -- config

