-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
--
-- The licensor cannot revoke these freedoms as long as you follow the
-- license terms.
--
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions --  You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

local RPTAGS = RPTAGS;

table.insert(RPTAGS.startup, function() -- startup function

  if RPTAGS.mode == "standalone" or RPTAGS.mode == "parallel"
  then 
    local CreateFrame = CreateFrame;
  
    local fontFrame = CreateFrame('frame');
    local font = fontFrame:CreateFontString(nil, 'OVERLAY', 'GameFontNormal');
    local FONTFILE, FONTSIZE, _ = font:GetFont();

    local RP_TagsDB = RP_TagsDB;
  
    local oUF = RPTAGS.oUF;
  
    local Config    = RPTAGS.utils.config;
    local getLayout    = RPTAGS.utils.rpuf.frame.layout;
    local Left      = RPTAGS.utils.rpuf.element.left;
    local Top       = RPTAGS.utils.rpuf.element.top;
    local Height    = RPTAGS.utils.rpuf.element.height;
    local Width     = RPTAGS.utils.rpuf.element.width;
    local Framesize = RPTAGS.utils.rpuf.frame.size;
    local MoveAll   = RPTAGS.utils.rpuf.frame.moveall;
    local IP = RPTAGS.const.RPUF.INITIAL_POSITION;
    local loc       = RPTAGS.utils.locale.loc;
  
    if not RPTAGS.utils      then RPTAGS.utils      = {} end;
    if not RPTAGS.utils.rpuf then RPTAGS.utils.rpuf = {} end;
  
    local TTwidth, TTborder, TTpadding = 200, 5, 5;
  
    local Tooltip = CreateFrame("Frame", "RPUF_Tooltip", UIParent)
          Tooltip:SetWidth(TTwidth);
          Tooltip:SetHeight(1)
          Tooltip:SetPoint("CENTER");
          Tooltip:SetBackdrop(RPTAGS.const.BACKDROP.BLIZZTOOLTIP)
          Tooltip:SetBackdropColor(0, 0, 0, 1)
          Tooltip:SetFrameStrata("TOOLTIP")
          Tooltip:Hide();
  
    function Tooltip:UpdateColors()
      local bgRed, bgGreen, bgBlue = RPTAGS.utils.color.hexaToNumber(Config.get("COLOR_RPUF"));
      self:SetBackdropColor(bgRed / 511, bgGreen / 511, bgBlue /511, 0.5 + Config.get("RPUFALPHA") / 200)
    end;
         
          Tooltip:UpdateColors();
    local TooltipPanel = CreateFrame('Frame', nil, Tooltip) 
          TooltipPanel:SetFrameLevel(Tooltip:GetFrameLevel() + 1) 
          TooltipPanel:SetWidth(TTwidth);
          TooltipPanel:SetHeight(1)
          TooltipPanel:SetPoint("TOPLEFT", Tooltip, "TOPLEFT", 0, 0)
          Tooltip.panel = TooltipPanel;
  
    local function RPUF_Panel_OnEnter(self) 
      SetCursor("INTERACT_CURSOR"); -- silver gear
      if Config.get(self.ttsource):match("^[ \n]*$") then return end; -- don't show the tooltip
      self.tooltip:SetPoint("TOPLEFT", Tooltip.panel, "TOPLEFT", TTborder + TTpadding, -1*(TTborder + TTpadding))
      Tooltip:SetHeight(2*TTborder + 2*TTpadding + self.tooltip:GetStringHeight());
      Tooltip.panel:SetHeight(2*TTborder + 2*TTpadding + self.tooltip:GetStringHeight());
      Tooltip:SetPoint("BOTTOMRIGHT", self, "TOPRIGHT", 0, 0);
      Tooltip:Show(); 
    end; -- function
  
    local function RPUF_Panel_OnLeave(self)
      SetCursor(nil);
      self.tooltip:SetPoint("TOPLEFT", Tooltip.panel, "TOPLEFT", 5000, 5000)
      Tooltip:Hide(); 
      end;
  
    local function RPUF_Panel_OnMouseUp(self, button)
      if   button == "RightButton" 
      then local contextMenuFrame = CreateFrame("Frame", "RPUF_Panel_ContextMenu", UIParent, "UIDropDownMenuTemplate");
           local contextMenu = { { text = loc("CONTEXT_MENU_TITLE"), isTitle = true, } };
  
           if   self.source 
           then table.insert(contextMenu, 
                  { text = RPTAGS.utils.text.hilite(loc("EDIT_TAGS_FOR").. "[[[" .. loc("CONFIG_"..self.source) .. "]]]"),
                    func = function() RPTAGS.utils.rpuf.tagEdit(self.source) 
                                      contextMenuFrame:Hide();
                                      end })
           end; -- if source
  
           if   self.ttsource
           then table.insert(contextMenu, 
                  { text = RPTAGS.utils.text.hilite(loc("EDIT_TAGS_FOR").."[["..loc("CONFIG_"..self.ttsource) .."]]"),
                    func = function() RPTAGS.utils.rpuf.tagEdit(self.ttsource) 
                                      contextMenuFrame:Hide();
                                      end })
           end; -- if ttsource
  
           if self.contextmenu 
           then for _, item in ipairs(self.contextmenu)
                do table.insert(contextMenu, item)
                end; -- for
           end; -- if contextmenu
  
           local unitFrame = self:GetParent();
           local unitFrameName = unitFrame:GetName();
           local unitFrameContextMenu = {};
  
           local layoutOpts = RPTAGS.cache.layoutOptions[unitFrameName]
           local currentSetting = RPTAGS.utils.config.get(layoutOpts.config);
  
           for _, opts in ipairs(layoutOpts.choices)
           do  label, value = unpack(opts);
               table.insert(unitFrameContextMenu, 
               { text    = label, 
                 value   = value,
                 checked = value == currentSetting,
                 func    = function(self) 
                             RPTAGS.utils.config.set(layoutOpts.config, self.value);
                             RPTAGS.utils.rpuf.frame.refreshall();
                             TRP3_API.configuration.refreshPage("main_config_rptags_ddd_rpuf_layout");
                             contextMenuFrame:Hide();
                           end }); -- func
           end; -- for opts
  
           table.insert(contextMenu, 
             { text = RPTAGS.utils.text.hilite(loc("SET_LAYOUT") .. 
                        "[[[" .. loc("CONFIG_" .. RPTAGS.cache.layoutOptions[unitFrameName].config) .. "]]]"),
               hasArrow = true,
               menuList = unitFrameContextMenu })
  
           table.insert(contextMenu, 
             { text = loc("CONFIGURE_FRAME_LAYOUT"),
               func = function() 
                        TRP3_API.navigation.menu.selectMenu("main_90_config");
                        TRP3_API.navigation.page.setPage("main_config_rptags_ddd_rpuf_layout");
                        TRP3_API.navigation.openMainFrame();
                        contextMenuFrame:Hide()
                      end });

           if   Config.get("RPUF_HIDE")
           then table.insert(contextMenu, 
                  { text = loc("CONTEXT_SHOW_FRAMES"),   
                    func = function(self) Config.set("RPUF_HIDE", false); Config.refresh("RPUF_HIDE"); end });
           else table.insert(contextMenu, 
                  { text = loc("CONTEXT_HIDE_FRAMES"),
                    func = function(self) Config.set("RPUF_HIDE", true ); Config.refresh("RPUF_HIDE"); end });
           end;

           if   Config.get("LOCK_FRAMES")
           then table.insert(contextMenu, 
                  { text = loc("CONTEXT_UNLOCK_FRAMES"), 
                    func = function(self) Config.set("LOCK_FRAMES", false); Config.refresh("LOCK_FRAMES"); end });
           else table.insert(contextMenu, 
                  { text = loc("CONTEXT_LOCK_FRAMES"),   
                    func = function(self) Config.set("LOCK_FRAMES", true);  Config.refresh("LOCK_FRAMES"); end });
           end -- if

           table.insert(contextMenu, { text = loc("CANCEL"), func = function() contextMenuFrame:Hide() end });
  
           if   self.source or self.ttsource or self.contextmenu
           then EasyMenu(contextMenu, contextMenuFrame, "cursor", 0, 0, "MENU")
           end; -- if source or ttsource
         end; -- if rightbutton
    end; -- function
  

    local function RPUF_Style(self, unit)

      local fix = RPTAGS.utils.rpuf.fixTagErrors;

      unit = unit:match('^(.-)%d+') or unit
  
      local layout = getLayout(self);
      local framewidth, frameheight = Framesize(layout);
  
      self.frameType = "RPUF";
  
      self:SetWidth( framewidth );
      self:SetHeight(frameheight);
      self:RegisterForClicks('AnyUp')
      self:SetScript('OnEnter', UnitFrame_OnEnter)
      self:SetScript('OnLeave', UnitFrame_OnLeave)
      self:SetClampedToScreen(true);

      local bgRed, bgGreen, bgBlue = RPTAGS.utils.color.hexaToNumber(Config.get("COLOR_RPUF"));
  
      self:SetBackdrop(RPTAGS.const.BACKDROP[RPTAGS.utils.config.get("RPUF_BACKDROP")])
      self:SetBackdropColor(bgRed / 255, bgGreen / 255, bgBlue / 255, Config.get("RPUFALPHA") / 100)
  
      local NamePanel = CreateFrame('Frame', nil, self) NamePanel:SetFrameLevel(20) self.NamePanel = NamePanel;
            NamePanel:SetPoint('TOPLEFT', self, 'TOPLEFT', Left('NamePanel', layout), Top('NamePanel', layout))
            NamePanel:SetSize(Width('NamePanel', layout), Height('NamePanel', layout));
            
      local NameFontString = self.NamePanel:CreateFontString(nil, 'OVERLAY', 'GameFontHighlightLarge')
            NamePanel.text = NameFontString;
            NameFontString:SetJustifyH('LEFT')
            NameFontString:SetJustifyV('TOP')
            NameFontString:SetSize(Width('NamePanel', layout), Height('NamePanel', layout));
            NameFontString:SetFont(FONTFILE, FONTSIZE + 4);
            NameFontString:SetPoint("TOPLEFT", NamePanel, "TOPLEFT", 0, 0);
            self:Tag(NameFontString, fix(Config.get("NAMEPANEL")))
      local NameTooltip       = Tooltip.panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            NamePanel.tooltip = NameTooltip;
            NameTooltip:SetPoint("TOPLEFT", Tooltip.panel, "TOPLEFT", 5000, 5000)
            NameTooltip:SetWidth(TTwidth - 2*TTborder - 2*TTpadding);
            NameTooltip:SetJustifyH('LEFT')
            NameTooltip:SetJustifyV('TOP')
            NameTooltip:SetWordWrap(true);
            NamePanel.ttsource = "NAME_TOOLTIP"
            self:Tag(NameTooltip, fix(Config.get(NamePanel.ttsource)))
            NamePanel:SetScript("OnEnter", RPUF_Panel_OnEnter);
            NamePanel:SetScript("OnLeave", RPUF_Panel_OnLeave);
            NamePanel.source = "NAMEPANEL";
            NamePanel:SetScript("OnMouseUp", RPUF_Panel_OnMouseUp);
            -- ---------------------------------------------------------------------------------------------------------
  
      local InfoPanel = CreateFrame('Frame', nil, self) InfoPanel:SetFrameLevel(20) self.InfoPanel = InfoPanel;
            InfoPanel:SetPoint('TOPLEFT', self, 'TOPLEFT', Left('InfoPanel', layout), Top('InfoPanel', layout))
            InfoPanel:SetSize(Width('InfoPanel', layout), Height('InfoPanel', layout));
  
      local InfoFontString = self.InfoPanel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            InfoPanel.text = InfoFontString;
            InfoFontString:SetPoint("TOPLEFT", InfoPanel, "TOPLEFT", 0, 0);
            InfoFontString:SetJustifyH('LEFT')
            InfoFontString:SetJustifyV('TOP')
            InfoFontString:SetWidth(Width('InfoPanel', layout));
            InfoFontString:SetHeight(Height('InfoPanel', layout));
            InfoFontString:SetFont(FONTFILE, FONTSIZE + 2);
            self:Tag(InfoFontString, fix(Config.get("INFOPANEL")))
      local InfoTooltip       = Tooltip.panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            InfoPanel.tooltip = InfoTooltip;
            InfoTooltip:SetPoint("TOPLEFT", Tooltip.panel, "TOPLEFT", 5000, 5000)
            InfoTooltip:SetWidth(TTwidth - 2*TTborder - 2*TTpadding);
            InfoTooltip:SetJustifyH('LEFT')
            InfoTooltip:SetJustifyV('TOP')
            InfoTooltip:SetWordWrap(true);
            InfoPanel.ttsource = "INFO_TOOLTIP"
            self:Tag(InfoTooltip, fix(Config.get(InfoPanel.ttsource)))
            InfoPanel:SetScript("OnEnter", RPUF_Panel_OnEnter);
            InfoPanel:SetScript("OnLeave", RPUF_Panel_OnLeave);
            InfoPanel.source = "INFOPANEL";
            InfoPanel:SetScript("OnMouseUp", RPUF_Panel_OnMouseUp);
            -- ---------------------------------------------------------------------------------------------------------
  
      local DetailsPanel = CreateFrame('Frame', nil, self) DetailsPanel:SetFrameLevel(20) self.DetailsPanel = DetailsPanel;
            DetailsPanel:SetPoint('TOPLEFT', self, 'TOPLEFT', Left('DetailsPanel', layout), Top('DetailsPanel', layout));
            DetailsPanel:SetSize(Width('DetailsPanel', layout), Height('DetailsPanel', layout));
  
      local DetailsFontString = self.DetailsPanel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal');
            DetailsPanel.text = DetailsFontString;
            DetailsFontString:SetPoint("TOPLEFT", DetailsPanel, "TOPLEFT", 0, 0);
            DetailsFontString:SetJustifyH('LEFT');
            DetailsFontString:SetJustifyV('TOP');
            DetailsFontString:SetWordWrap(true);
            DetailsFontString:SetWidth(Width('DetailsPanel', layout));
            DetailsFontString:SetHeight(Height('DetailsPanel', layout));
            self:Tag(DetailsFontString, fix(Config.get("DETAILPANEL")));
      local DetailsTooltip       = Tooltip.panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            DetailsPanel.tooltip = DetailsTooltip;
            DetailsTooltip:SetPoint("TOPLEFT", Tooltip.panel, "TOPLEFT", 5000, 5000)
            DetailsTooltip:SetWidth(TTwidth - 2*TTborder - 2*TTpadding);
            DetailsTooltip:SetJustifyH('LEFT')
            DetailsTooltip:SetJustifyV('TOP')
            DetailsTooltip:SetWordWrap(true);
            DetailsPanel.ttsource = "DETAIL_TOOLTIP"
            self:Tag(DetailsTooltip, fix(Config.get(DetailsPanel.ttsource)))
            DetailsPanel:SetScript("OnEnter", RPUF_Panel_OnEnter);
            DetailsPanel:SetScript("OnLeave", RPUF_Panel_OnLeave);
            DetailsPanel.source = "DETAILPANEL";
            DetailsPanel:SetScript("OnMouseUp", RPUF_Panel_OnMouseUp);
            -- ---------------------------------------------------------------------------------------------------------
  
      local Portrait = CreateFrame('PlayerModel', nil, self)
            Portrait:SetPoint("TOPLEFT", self, 'TOPLEFT', Left('Portrait', layout), Top('Portrait', layout));
            Portrait:SetWidth(Width('Portrait', layout))
            Portrait:SetHeight(Height('Portrait', layout))
            Portrait:SetFrameLevel(self:GetFrameLevel() + 1)
            self.Portrait = Portrait;
      local PortraitTooltip       = Tooltip.panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            Portrait.tooltip = PortraitTooltip;
            PortraitTooltip:SetPoint("TOPLEFT", Tooltip.panel, "TOPLEFT", 5000, 5000)
            PortraitTooltip:SetWidth(TTwidth - 2*TTborder - 2*TTpadding);
            PortraitTooltip:SetJustifyH('LEFT')
            PortraitTooltip:SetJustifyV('TOP')
            PortraitTooltip:SetWordWrap(true);
            Portrait.ttsource = "PORTRAIT_TOOLTIP"
            self:Tag(PortraitTooltip, fix(Config.get(Portrait.ttsource)))
            Portrait:SetScript("OnEnter", RPUF_Panel_OnEnter);
            Portrait:SetScript("OnLeave", RPUF_Panel_OnLeave);
            -- Portrait.contextmenu = { { text = loc("PORTRAIT_TYPE_FACE").." "..loc("CONFIG_PORTRAIT_TYPE"),
            --                            func = function() Config.set("PORTRAIT_TYPE", "FACE") end },
            --                          { text = loc("PORTRAIT_TYPE_FULLBODY").." "..loc("CONFIG_PORTRAIT_TYPE"),
            --                            func = function() Config.set("PORTRAIT_TYPE", "FULLBODY") end },
            --                          };
            Portrait:SetScript("OnMouseUp", RPUF_Panel_OnMouseUp);
            -- ---------------------------------------------------------------------------------------------------------
  
      local Icon_1Panel = CreateFrame('Frame', nil, self) Icon_1Panel:SetFrameLevel(20) self.Icon_1Panel = Icon_1Panel;
            Icon_1Panel:SetPoint('TOPLEFT', self, 'TOPLEFT', Left('Icon_1Panel', layout), Top('Icon_1Panel', layout))
            Icon_1Panel:SetSize(Width('Icon_1Panel', layout), Height('Icon_1Panel', layout));
  
      local Icon_1FontString = self.Icon_1Panel:CreateFontString(nil, 'OVERLAY', 'GameFontHighlightLarge')
            Icon_1Panel.text = Icon_1FontString;
            Icon_1FontString:SetPoint("TOPLEFT", Icon_1Panel, "TOPLEFT", 0, 0);
            Icon_1FontString:SetJustifyH('LEFT')
            Icon_1FontString:SetJustifyV('TOP')
            Icon_1FontString:SetWordWrap(false)
            Icon_1FontString:SetFont(FONTFILE, Width('Icon_1Panel', layout) -1 )
            self:Tag(Icon_1FontString, fix(Config.get("ICON_1")))
  
      local Icon_1Tooltip       = Tooltip.panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            Icon_1Panel.tooltip = Icon_1Tooltip;
            Icon_1Tooltip:SetPoint("TOPLEFT", Tooltip.panel, "TOPLEFT", 5000, 5000)
            Icon_1Tooltip:SetWidth(TTwidth - 2*TTborder - 2*TTpadding);
            Icon_1Tooltip:SetJustifyH('LEFT')
            Icon_1Tooltip:SetJustifyV('TOP')
            Icon_1Tooltip:SetWordWrap(true);
            Icon_1Panel.ttsource = "ICON_1_TOOLTIP"
            self:Tag(Icon_1Tooltip, fix(Config.get(Icon_1Panel.ttsource)))
            Icon_1Panel:SetScript("OnEnter", RPUF_Panel_OnEnter);
            Icon_1Panel:SetScript("OnLeave", RPUF_Panel_OnLeave);
            Icon_1Panel.source = "ICON_1";
            Icon_1Panel:SetScript("OnMouseUp", RPUF_Panel_OnMouseUp);
            -- ---------------------------------------------------------------------------------------------------------
  
      local Icon_2Panel = CreateFrame('Frame', nil, self) Icon_2Panel:SetFrameLevel(20) self.Icon_2Panel = Icon_2Panel;
            Icon_2Panel:SetPoint('TOPLEFT', self, 'TOPLEFT', Left('Icon_2Panel', layout), Top('Icon_2Panel', layout));
            Icon_2Panel:SetSize(Width('Icon_2Panel', layout), Height('Icon_2Panel', layout));
  
      local Icon_2FontString = self.Icon_2Panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            Icon_2Panel.text = Icon_2FontString;
            Icon_2FontString:SetPoint("TOPLEFT", Icon_2Panel, "TOPLEFT", 0, 0);
            Icon_2FontString:SetJustifyH('LEFT')
            Icon_2FontString:SetJustifyV('TOP')
            Icon_2FontString:SetWordWrap(false)
            Icon_2FontString:SetFont(FONTFILE, Width('Icon_2Panel', layout) -1 )
            self:Tag(Icon_2FontString, fix(Config.get("ICON_2")))
  
      local Icon_2Tooltip       = Tooltip.panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            Icon_2Panel.tooltip = Icon_2Tooltip;
            Icon_2Tooltip:SetPoint("TOPLEFT", Tooltip.panel, "TOPLEFT", 5000, 5000)
            Icon_2Tooltip:SetWidth(TTwidth - 2*TTborder - 2*TTpadding);
            Icon_2Tooltip:SetJustifyH('LEFT')
            Icon_2Tooltip:SetJustifyV('TOP')
            Icon_2Tooltip:SetWordWrap(true);
            Icon_2Panel.ttsource = "ICON_2_TOOLTIP"
            self:Tag(Icon_2Tooltip, fix(Config.get(Icon_2Panel.ttsource)))
            Icon_2Panel:SetScript("OnEnter", RPUF_Panel_OnEnter);
            Icon_2Panel:SetScript("OnLeave", RPUF_Panel_OnLeave);
            Icon_2Panel.source = "ICON_2";
            Icon_2Panel:SetScript("OnMouseUp", RPUF_Panel_OnMouseUp);
            -- ---------------------------------------------------------------------------------------------------------
      local Icon_3Panel = CreateFrame('Frame', nil, self) Icon_3Panel:SetFrameLevel(20) self.Icon_3Panel = Icon_3Panel;
            Icon_3Panel:SetPoint('TOPLEFT', self, 'TOPLEFT', Left('Icon_3Panel', layout), Top('Icon_3Panel', layout));
            Icon_3Panel:SetSize(Width('Icon_3Panel', layout), Height('Icon_3Panel', layout));
  
      local Icon_3FontString = self.Icon_3Panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            Icon_3Panel.text = Icon_3FontString;
            Icon_3FontString:SetPoint("TOPLEFT", Icon_3Panel, "TOPLEFT", 0, 0);
            Icon_3FontString:SetJustifyH('LEFT')
            Icon_3FontString:SetJustifyV('TOP')
            Icon_3FontString:SetWordWrap(false)
            Icon_3FontString:SetFont(FONTFILE, Width('Icon_3Panel', layout) -1 )
            self:Tag(Icon_3FontString, fix(Config.get("ICON_3")))
  
      local Icon_3Tooltip       = Tooltip.panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            Icon_3Panel.tooltip = Icon_3Tooltip;
            Icon_3Tooltip:SetPoint("TOPLEFT", Tooltip.panel, "TOPLEFT", 5000, 5000)
            Icon_3Tooltip:SetWidth(TTwidth - 2*TTborder - 2*TTpadding);
            Icon_3Tooltip:SetJustifyH('LEFT')
            Icon_3Tooltip:SetJustifyV('TOP')
            Icon_3Tooltip:SetWordWrap(true);
            Icon_3Panel.ttsource = "ICON_3_TOOLTIP"
            self:Tag(Icon_3Tooltip, fix(Config.get(Icon_3Panel.ttsource)))
            Icon_3Panel:SetScript("OnEnter", RPUF_Panel_OnEnter);
            Icon_3Panel:SetScript("OnLeave", RPUF_Panel_OnLeave);
            Icon_3Panel.source = "ICON_3";
            Icon_3Panel:SetScript("OnMouseUp", RPUF_Panel_OnMouseUp);
            -- ---------------------------------------------------------------------------------------------------------
      local Icon_4Panel = CreateFrame('Frame', nil, self) Icon_4Panel:SetFrameLevel(20) self.Icon_4Panel = Icon_4Panel;
            Icon_4Panel:SetPoint('TOPLEFT', self, 'TOPLEFT', Left('Icon_4Panel', layout), Top('Icon_4Panel', layout));
            Icon_4Panel:SetSize(Width('Icon_4Panel', layout), Height('Icon_4Panel', layout));
  
      local Icon_4FontString = self.Icon_4Panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            Icon_4Panel.text = Icon_4FontString;
            Icon_4FontString:SetPoint("TOPLEFT", Icon_4Panel, "TOPLEFT", 0, 0);
            Icon_4FontString:SetJustifyH('LEFT')
            Icon_4FontString:SetJustifyV('TOP')
            Icon_4FontString:SetWordWrap(false)
            Icon_4FontString:SetFont(FONTFILE, Width('Icon_4Panel', layout) -1 )
            self:Tag(Icon_4FontString, fix(Config.get("ICON_4")))
  
      local Icon_4Tooltip       = Tooltip.panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            Icon_4Panel.tooltip = Icon_4Tooltip;
            Icon_4Tooltip:SetPoint("TOPLEFT", Tooltip.panel, "TOPLEFT", 5000, 5000)
            Icon_4Tooltip:SetWidth(TTwidth - 2*TTborder - 2*TTpadding);
            Icon_4Tooltip:SetJustifyH('LEFT')
            Icon_4Tooltip:SetJustifyV('TOP')
            Icon_4Tooltip:SetWordWrap(true);
            Icon_4Panel.ttsource = "ICON_4_TOOLTIP"
            self:Tag(Icon_4Tooltip, fix(Config.get(Icon_4Panel.ttsource)))
            Icon_4Panel:SetScript("OnEnter", RPUF_Panel_OnEnter);
            Icon_4Panel:SetScript("OnLeave", RPUF_Panel_OnLeave);
            Icon_4Panel.source = "ICON_4";
            Icon_4Panel:SetScript("OnMouseUp", RPUF_Panel_OnMouseUp);
            -- ---------------------------------------------------------------------------------------------------------
      local Icon_5Panel = CreateFrame('Frame', nil, self) Icon_5Panel:SetFrameLevel(20) self.Icon_5Panel = Icon_5Panel;
            Icon_5Panel:SetPoint('TOPLEFT', self, 'TOPLEFT', Left('Icon_5Panel', layout), Top('Icon_5Panel', layout));
            Icon_5Panel:SetSize(Width('Icon_5Panel', layout), Height('Icon_5Panel', layout));
  
      local Icon_5FontString = self.Icon_5Panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            Icon_5Panel.text = Icon_5FontString;
            Icon_5FontString:SetPoint("TOPLEFT", Icon_5Panel, "TOPLEFT", 0, 0);
            Icon_5FontString:SetJustifyH('LEFT')
            Icon_5FontString:SetJustifyV('TOP')
            Icon_5FontString:SetWordWrap(false)
            Icon_5FontString:SetFont(FONTFILE, Width('Icon_5Panel', layout) -1 )
            self:Tag(Icon_5FontString, fix(Config.get("ICON_5")))
  
      local Icon_5Tooltip       = Tooltip.panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            Icon_5Panel.tooltip = Icon_5Tooltip;
            Icon_5Tooltip:SetPoint("TOPLEFT", Tooltip.panel, "TOPLEFT", 5000, 5000)
            Icon_5Tooltip:SetWidth(TTwidth - 2*TTborder - 2*TTpadding);
            Icon_5Tooltip:SetJustifyH('LEFT')
            Icon_5Tooltip:SetJustifyV('TOP')
            Icon_5Tooltip:SetWordWrap(true);
            Icon_5Panel.ttsource = "ICON_5_TOOLTIP"
            self:Tag(Icon_5Tooltip, fix(Config.get(Icon_5Panel.ttsource)))
            Icon_5Panel:SetScript("OnEnter", RPUF_Panel_OnEnter);
            Icon_5Panel:SetScript("OnLeave", RPUF_Panel_OnLeave);
            Icon_5Panel.source = "ICON_5";
            Icon_5Panel:SetScript("OnMouseUp", RPUF_Panel_OnMouseUp);
            -- ---------------------------------------------------------------------------------------------------------
      local Icon_6Panel = CreateFrame('Frame', nil, self) Icon_6Panel:SetFrameLevel(20) self.Icon_6Panel = Icon_6Panel;
            Icon_6Panel:SetPoint('TOPLEFT', self, 'TOPLEFT', Left('Icon_6Panel', layout), Top('Icon_6Panel', layout));
            Icon_6Panel:SetSize(Width('Icon_6Panel', layout), Height('Icon_6Panel', layout));
          
      local Icon_6FontString = self.Icon_6Panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            Icon_6Panel.text = Icon_6FontString;
            Icon_6FontString:SetPoint("TOPLEFT", Icon_6Panel, "TOPLEFT", 0, 0);
            Icon_6FontString:SetJustifyH('LEFT')
            Icon_6FontString:SetJustifyV('TOP')
            Icon_6FontString:SetWordWrap(false)
            Icon_6FontString:SetFont(FONTFILE, Width('Icon_6Panel', layout) -1 )
            self:Tag(Icon_6FontString, fix(Config.get("ICON_6")))
  
      local Icon_6Tooltip       = Tooltip.panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            Icon_6Panel.tooltip = Icon_6Tooltip;
            Icon_6Tooltip:SetPoint("TOPLEFT", Tooltip.panel, "TOPLEFT", 5000, 5000)
            Icon_6Tooltip:SetWidth(TTwidth - 2*TTborder - 2*TTpadding);
            Icon_6Tooltip:SetJustifyH('LEFT')
            Icon_6Tooltip:SetJustifyV('TOP')
            Icon_6Tooltip:SetWordWrap(true);
            Icon_6Panel.ttsource = "ICON_6_TOOLTIP"
            self:Tag(Icon_6Tooltip, fix(Config.get(Icon_6Panel.ttsource)))
            Icon_6Panel:SetScript("OnEnter", RPUF_Panel_OnEnter);
            Icon_6Panel:SetScript("OnLeave", RPUF_Panel_OnLeave);
            Icon_6Panel.source = "ICON_6";
            Icon_6Panel:SetScript("OnMouseUp", RPUF_Panel_OnMouseUp);
            -- ---------------------------------------------------------------------------------------------------------
      local StatusBarPanel = CreateFrame('Frame', nil, self) StatusBarPanel:SetFrameLevel(20) self.StatusBarPanel = StatusBarPanel;
            StatusBarPanel:SetPoint('TOPLEFT', self, 'TOPLEFT', Left('StatusBarPanel', layout), Top('StatusBarPanel', layout));
            StatusBarPanel:SetSize(Width('StatusBarPanel', layout), Height('StatusBarPanel', layout));
  
      local StatusBarFontString = self.StatusBarPanel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            StatusBarPanel.text = StatusBarFontString;
            StatusBarFontString:SetJustifyH('CENTER');
            StatusBarFontString:SetPoint("TOPLEFT", StatusBarPanel, "TOPLEFT", 0, 0);
            StatusBarFontString:SetJustifyV('TOP');
            StatusBarFontString:SetWordWrap(true);
            StatusBarFontString:SetSize(Width('StatusBarPanel', layout), Height('StatusBarPanel', layout));
            self:Tag(StatusBarFontString, fix(Config.get("STATUSPANEL")))
      local StatusBarTooltip       = Tooltip.panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
            StatusBarPanel.tooltip = StatusBarTooltip;
            StatusBarTooltip:SetPoint("TOPLEFT", Tooltip.panel, "TOPLEFT", 5000, 5000)
            StatusBarTooltip:SetWidth(TTwidth - 2*TTborder - 2*TTpadding);
            StatusBarTooltip:SetJustifyH('LEFT')
            StatusBarTooltip:SetJustifyV('TOP')
            StatusBarTooltip:SetWordWrap(true);
            StatusBarPanel.ttsource = "STATUS_TOOLTIP"
            self:Tag(StatusBarTooltip, fix(Config.get(StatusBarPanel.ttsource)))
            StatusBarPanel:SetScript("OnEnter", RPUF_Panel_OnEnter);
            StatusBarPanel:SetScript("OnLeave", RPUF_Panel_OnLeave);
            StatusBarPanel.source = "STATUSPANEL";
            StatusBarPanel:SetScript("OnMouseUp", RPUF_Panel_OnMouseUp);
            -- ---------------------------------------------------------------------------------------------------------
            
      local DragFrame = CreateFrame("frame", nil, self)
            self.DragFrame = DragFrame;
            DragFrame:SetBackdrop(RPTAGS.const.BACKDROP.BLIZZTOOLTIP)
            DragFrame:SetBackdropColor(0, 0, 0, 1)
            DragFrame:SetSize(50, 72);
            DragFrame:SetPoint("TOPRIGHT", self, "TOPLEFT", 5, 0)

            DragFrame:SetScript("OnMouseDown",
              function(self)
                self:GetParent():SetMovable(true);
                self:GetParent():EnableMouse(true);
                self:GetParent():StartMoving();
              end);
            DragFrame:SetScript("OnMouseUp",
              function(self)
                self:GetParent():SetMovable(false);
                self:GetParent():EnableMouse(false);
                self:GetParent():StopMovingOrSizing();
                local point, relativeTo, relativePoint, xOfs, yOfs = self:GetParent():GetPoint()
                if relativeTo == nil then relativeTo = "UIParent" else relativeTo = relativeTo:GetName(); end;
                RP_TagsDB[self:GetParent().unit .. "UFlocation"] = { point, relativeTo, relativePoint, xOfs, yOfs };
              end);
            if Config.get("LOCK_FRAMES") then DragFrame:Hide() end;

      local DragFrameArrows = DragFrame:CreateTexture();
            DragFrameArrows:SetTexture([[Interface\Cursor\UI-Cursor-Move]]);
            DragFrameArrows:SetSize(36,36);
            DragFrameArrows:SetPoint("TOP", DragFrame, "TOP", 0, -9);

      local DragFrameLockButton = CreateFrame("button", nil, DragFrame)
            DragFrameLockButton:SetSize(26,26);
            -- DragFrameLockButton:SetNormalTexture("Interface\\PetBattles\\PetBattle-LockIcon");
            DragFrameLockButton:SetNormalAtlas("Garr_LockedBuilding");
            DragFrameLockButton:SetPoint("BOTTOMRIGHT", DragFrame, "BOTTOMRIGHT", -1, 2);
            DragFrameLockButton:SetScript("OnClick", 
              function(self) 
                Config.set("LOCK_FRAMES", true);
                Config.refresh("LOCK_FRAMES");
              end);

      local DragFrameResetButton = CreateFrame("button", nil, DragFrame)
            DragFrameResetButton:SetSize(24,24);
            -- DragFrameResetButton:SetNormalTexture("Interface\\PaperDollInfoFrame\\UI-GearManager-Undo")
            DragFrameResetButton:SetNormalAtlas("transmog-icon-revert")
            DragFrameResetButton:SetPoint("BOTTOMLEFT", DragFrame, "BOTTOMLEFT", 4, 3);
            DragFrameResetButton:SetScript("OnClick", 
              function(self)
                local frame = self:GetParent():GetParent();
                local IP = RPTAGS.const.RPUF.INITIAL_POSITION[frame.unit];
                -- local w, h = RPTAGS.utils.rpuf.frame.size(frame);
                -- frame:SetSize(w, h);
                frame:ClearAllPoints();
                frame:SetPoint(IP.pt, _G[IP.relto], IP.relpt, IP.x, IP.y);
                RP_TagsDB[frame.unit .. "UFlocation"] = nil;
              end);

            -- ---------------------------------------------------------------------------------------------------------
    end;
  
    local function enableBlizzard() function oUF:DisableBlizzard() return self or true; end; end;
    local function disableBlizzard() return nil; end;
    if RP_TagsDB.DISABLE_BLIZZARD then disableBlizzard() else enableBlizzard() end; -- if
  
    oUF:RegisterStyle('RP_Tags', RPUF_Style)
  
    oUF:Factory(
        function(self)
  
          self:SetActiveStyle('RP_Tags')
          local UF = {};
  
          UF.p = RP_TagsDB.playerUFlocation or { IP.player.pt, _G[IP.player.relto], IP.player.relpt, IP.player.x, IP.player.y };
          self:Spawn('player', "RPUF_Player"):SetPoint(unpack(UF.p));
  
          UF.f = RP_TagsDB.focusUFlocation or { IP.focus.pt, _G[IP.focus.relto], IP.focus.relpt, IP.focus.x, IP.focus.y };
          self:Spawn('focus', "RPUF_Focus"):SetPoint(unpack(UF.f));
  
          UF.t = RP_TagsDB.targetUFlocation or { IP.target.pt, _G[IP.target.relto], IP.target.relpt, IP.target.x, IP.target.y };
          self:Spawn('target', "RPUF_Target"):SetPoint(unpack(UF.t));
  
          UF.tt = RP_TagsDB.targettargetUFlocation or { IP.targettarget.pt, _G[IP.targettarget.relto], IP.targettarget.relpt, IP.targettarget.x, IP.targettarget.y };
          self:Spawn('targettarget', "RPUF_TargetTarget"):SetPoint(unpack(UF.tt));
  
          RPUF_Player:Disable(); RPUF_Target:Disable(); RPUF_Focus:Disable(); RPUF_TargetTarget:Disable();
          RPTAGS.utils.rpuf.frame.sethide(true); -- true here means "initialization, i.e. it hasn't been run before

        end) -- function(self) and Factory
  
  --   self:SpawnHeader(nil, nil, 'custom [group:party] show; [@raid3,exists] show; [@raid26,exists] hide; hide',
  --                    'showParty',                 true,
  --                    'showRaid',                  true,
  --                    'showPlayer',                true,
  --                    'yOffset',                   -6,
  --                    'groupBy',                   'ASSIGNEDROLE',
  --                    'groupingOrder',             'TANK,HEALER,DAMAGER',
  --                    'oUF-initialConfigFunction', [[ self:SetHeight(19)
  --                                                    self:SetWidth(126) ]]
  --                   ):SetPoint('TOP', Minimap, 'BOTTOM', 0, -10)
  -- --
  --  for index = 1, 5
  --    do local boss  = self:Spawn('boss'  .. index)
  --       local arena = self:Spawn('arena' .. index)
  --
  --       if index == 1
  --          then boss:SetPoint( 'TOP', oUF_P3limRaid,                     'BOTTOM', 0, -20)
  --               arena:SetPoint('TOP', oUF_P3limRaid,                     'BOTTOM', 0, -20)
  --          else boss:SetPoint( 'TOP', _G['oUF_P3limBoss' .. index - 1 ], 'BOTTOM', 0,  -6)
  --               arena:SetPoint('TOP', _G['oUF_P3limArena' .. index - 1], 'BOTTOM', 0,  -6)
  --       end -- if index
  --    end -- do
  
  end; -- if

  return "mod-rpuf (" .. RPTAGS.mode .. "-mode)"
  
end); -- startup function
