local RPTAGS      = RPTAGS;

table.insert(RPTAGS.startup, function() -- startup function
  local fontFrame = CreateFrame('frame');
  local font = fontFrame:CreateFontString(nil, 'OVERLAY', 'GameFontNormal');
  local FONTFILE, _, _ = font:GetFont();
  
  local Const       = RPTAGS.const;
  local Utils       = RPTAGS.utils;
  local Config      = RPTAGS.utils.config;
  local loc         = RPTAGS.utils.locale.loc;
  local LSM         = LibStub("LibSharedMedia-3.0");
  local CW          = Const.icons.COLORWHEEL;
  local Ora         = RPTAGS.utils.text.ora;
  local notify      = RPTAGS.utils.text.notify;
  
  if not RPTAGS.utils then RPTAGS.utils = {}; end;
  if not RPTAGS.utils.rpuf then RPTAGS.utils.rpuf = {}; end;
  
  local function hiliteError(editBox, badTag)
    local startHilite     = string.find(editBox:GetText(), badTag) - 1;
    local endHilite       = startHilite + string.len(badTag) 
    editBox:HighlightText(startHilite, endHilite);
  end; -- function
    
  local TagEdit = CreateFrame("frame", "RPTAGS_TagEdit", UIParent, "BasicFrameTemplate");
        TagEdit:SetWidth(500);
        TagEdit:SetHeight(400);
        TagEdit:SetPoint("CENTER");
        TagEdit:SetMovable(true)
        TagEdit:EnableMouse(true)
        TagEdit:RegisterForDrag("LeftButton")
        TagEdit:SetScript("OnDragStart", TagEdit.StartMoving)
        TagEdit:SetScript("OnDragStop", TagEdit.StopMovingOrSizing)
        TagEdit:SetToplevel(true);
        TagEdit:SetUserPlaced(true);
        TagEdit:SetFrameLevel(TRP3_MainFrame:GetFrameLevel() + 1);
        TagEdit:SetScript("OnShow", RPTAGS.utils.rpuf.updateTagEdit);
  
        tinsert(UISpecialFrames, TagEdit:GetName());
  
  local frameTitle = TagEdit:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        frameTitle:SetText(loc("APP_NAME") .. loc("TAG_EDITOR"));
        frameTitle:SetPoint("TOPLEFT", TagEdit, "TOPLEFT", 5, -5);
  
  local contentPane = CreateFrame("Frame", nil, TagEdit);
        contentPane:SetPoint("TOPLEFT", TagEdit, "TOPLEFT", 5, -25);
        contentPane:SetPoint("BOTTOMRIGHT", TagEdit, "BOTTOMRIGHT", -10, 20);
  
  local contentTitle = contentPane:CreateFontString("RPTAGS_TagEditTitle", "OVERLAY", "GameFontNormalLarge");
        contentTitle:SetText("");
        contentTitle:SetPoint("TOPLEFT", contentPane, "TOPLEFT", 5, -5);
        TagEdit.contentTitle = contentTitle;
      
  local displayBox = contentPane:CreateFontString("RPTAGS_TagEditDisplay", "OVERLAY", "GameFontNormal");
        displayBox:SetWidth(420);
        displayBox:SetHeight(40);
        displayBox:SetJustifyH("LEFT");
        displayBox:SetJustifyV("TOP");
        displayBox:SetText(loc("CONFIG_STATUSPANEL_TT"));
                       
        displayBox:SetPoint("TOPLEFT", contentTitle, "BOTTOMLEFT", 0, -10);
        TagEdit.displayBox = displayBox;
  
  local buttons = {  -- CW is the Color Wheel graphic
    {  -- row 1
  --    button        tag           width     2nd button            width
      { "name",       "[rp:name]",   55, }, { CW, "[rp:color]",       25 }, -- 1
      { "eyes",       "[rp:eyes]",   55, }, { CW, "[rp:eyecolor]",    25 }, -- 2
      { "class",      "[rp:class]"       },                                 -- 3
      { "icon",       "[rp:icon]"        },                                 -- 4
      { "height",     "[rp:height]",     },                                 -- 5
      { CW .. " off", "[nocolor]"        },                                 -- 6
    }, -- row 1
    {  -- row 2
      { "ic/ooc",     "[rp:status]", 55, }, { CW, "[rp:statuscolor]", 25 }, -- 1
      { "sex",        "[rp:gender]", 55, }, { CW, "[rp:gendercolor]", 25 }, -- 2
      { "race",       "[rp:race]",       },                                 -- 3
      { "age",        "[rp:age]",        },                                 -- 4
      { "body",       "[rp:body]",       },                                 -- 5
       --                                     (reserved for "More...")      -- 6
    }, -- row 2
  }; -- buttons
  
  local lastButton, lastButtonRow;
  
  for _, row in ipairs(buttons)
  do  for i, buttonData in ipairs(row)
      do  local title, value, width = unpack(buttonData);
          local button = CreateFrame("button", nil, contentPane, "UIPanelButtonTemplate");
          button:SetText(title)
          button.value = value;
          button.tooltipText = vaue;
          button:SetWidth(width or 80);
          button:SetScript("OnClick", function(self) TagEdit.textbox:Insert(self.value) end);
  
          if   not lastButton
          then button:SetPoint("TOPLEFT", displayBox, "BOTTOMLEFT", 0, -10);
               lastButtonRow = button;
          elseif i == 1 
          then button:SetPoint("TOPLEFT", lastButtonRow, "BOTTOMLEFT", 0, 0); 
               lastButtonRow = button;
          else button:SetPoint("LEFT",    lastButton, "RIGHT", 0, 0);
          end; -- if
          lastButton = button;
      end; -- for i,buttonData
  end;
  
  local dropDownData = {};
  local dropDownFrame = CreateFrame("Frame", "ExampleMenuFrame", contentPane, "UIDropDownMenuTemplate")
        dropDownFrame:SetPoint("TOPLEFT", moretagsButton, "TOPRIGHT", 0, 0);
  
  local tagsSoFar;
  local currentMenu;
  
  for i, tagData in ipairs(RPTAGS.cache.tagorder)
  do  local  tagType, tagTitle, tagValue = unpack(tagData);
      if     tagType == "submenu" and not tagsSoFar  -- it's a submenu but our first apparently
      then   currentMenu = tagTitle;                 -- save the title of this submenu
             tagsSoFar = {};                         -- start a clean list of tSF
      elseif tagType == "submenu" and currentMenu    -- it's a new submenu but not our first
      then   table.insert(dropDownData, { text     = currentMenu, 
                                          hasArrow = true, 
                                          menuList = tagsSoFar, });
                                                     -- let's insert the data we have so far into the menu
             currentMenu = tagTitle;                 -- then save the new submenu name
             tagsSoFar = {};                         -- then clear the tags
      elseif tagType == "sep"
      then   table.insert(tagsSoFar, { text = tagTitle or "", isTitle = true, });
      elseif tagType == "tag"                     -- it's a tag so insert it into tagsSoFar
      then   table.insert(tagsSoFar, { text  = tagTitle, 
                                       value = tagValue, 
                                       func  = function(self) 
                                               TagEdit.textbox:Insert(tagValue); 
                                               ToggleDropDownMenu(1, nil, dropDownFrame); 
                                             end, 
                                             });
      end; -- if
  end; -- for tagData
  
  table.insert(dropDownData, { text     = currentMenu, 
                               hasArrow = true, 
                               menuList = tagsSoFar }); -- do the final pass
                   
  local moretagsButton = CreateFrame("button", "RPTAGS_TagEdit_moretagsButton", contentPane, "UIPanelButtonTemplate");
        moretagsButton:SetText(loc("MORE_TAGS"));
        moretagsButton:SetWidth(80);
        moretagsButton:SetPoint("TOP", lastButton, "TOP", 0, 0);
        moretagsButton:SetPoint("RIGHT", contentPane, "RIGHT", 0, 0);
        moretagsButton:SetScript("OnClick", function() EasyMenu(dropDownData, dropDownFrame, moretagsButton, 75, 5, "not MENU") end);
  
  local textboxContainer = CreateFrame("Frame", "RPTAGS_TagEditTextbox", contentPane);
        textboxContainer:SetBackdrop({
          bgFile   = [[Interface\Buttons\WHITE8x8]],
          edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
          edgeSize = 14,
  	insets = {left = 3, right = 3, top = 3, bottom = 3},
        });
        textboxContainer:SetBackdropColor(0, 0, 0, 0.5)
        textboxContainer:SetSize(480,125);
        textboxContainer:SetBackdropBorderColor(0.3, 0.3, 0.3)
        textboxContainer:SetPoint("LEFT", lastButtonRow, "LEFT", 0, 0);
        textboxContainer:SetPoint("TOP", lastButtonRow, "BOTTOM", 0, -5);
  
  local textboxScroll = CreateFrame("ScrollFrame", nil, textboxContainer, "UIPanelScrollFrameTemplate");
        -- textboxScroll:SetSize(400, 125);
        textboxScroll:SetPoint("TOPLEFT", textboxContainer, "TOPLEFT", 5, -5);
        -- textboxScroll:SetPoint("BOTTOMRIGHT", contentPane, "BOTTOMRIGHT", -5, -50);
        textboxScroll:SetSize(449,115);
  
  local textbox = CreateFrame("editbox", nil, textboxScroll);
        textbox:SetMultiLine(true);
        textbox:SetSize(440,120);
  
  local monoFont = LSM:Fetch("font", "SourceCodePro Regular");
        if monoFont then textbox:SetFont(monoFont, 14); else Ora("|cffdd0000Ora! The font didn't load!") end;
  
        textbox:SetText("");
        textbox:SetPoint("TOP", textboxScroll, "TOP", 0, -20);
        textbox:SetAutoFocus(false)
        textbox:SetScript("OnEscapePressed", function(self) self:ClearFocus() end);
        textbox:SetScript("OnTabPressed", function(self) self:ClearFocus() end);
        textbox:SetCursorPosition(0)
        textbox:SetJustifyH("LEFT")
        textbox:SetJustifyV("TOP")
        textbox:SetScript("OnShow", function(self) self:SetFocus() end);
        textbox:SetTextInsets(2, 2, 2, 2);
        textboxScroll:SetScrollChild(textbox);
        TagEdit.textbox = textbox;
  
  local saveButton = CreateFrame("button", nil, contentPane, "UIPanelButtonTemplate");
        saveButton:SetText(loc("TAG_EDIT_UPDATE"));
        saveButton:SetWidth(90);
        saveButton:SetPoint("TOPLEFT", textboxContainer, "BOTTOMLEFT", 0, -5)
        saveButton:SetScript("OnClick", 
            function(self)
              local text = TagEdit.textbox:GetText();
              local err, good, bad = RPTAGS.utils.rpuf.testTags(text);
              if err
                 then StaticPopup_Show("RPTAGS_TAGEDIT_ERROR");
                      hiliteError(TagEdit.textbox, bad[1]);
                 else if TagEdit.setting == "STATUSPANEL" or TagEdit.setting == "DETAILPANEL" or TagEdit.setting:gmatch("_TOOLTIP")
                         then text = text:gsub("%[p%]", "\n\n"):gsub("%[br%]", "\n"):gsub("\n\n+", "%[p%]"):gsub("\n","%[br%]");
                         else text = text:gsub("\n", "");
                         end;
                      text = text:trim(); 
                      Config.set(TagEdit.setting, text);
                      TagEdit.draft = nil;
                      TagEdit:Hide();
                 end;
              end);
  
  local testButton = CreateFrame("button", nil, contentPane, "UIPanelButtonTemplate");
        testButton:SetText(loc("TAG_EDIT_TEST"));
        testButton:SetWidth(90);
        testButton:SetPoint("LEFT", saveButton, "RIGHT", 7, 0)
        testButton:SetScript("OnClick", 
            function(self) 
              local err, good, bad = RPTAGS.utils.rpuf.testTags(TagEdit.textbox:GetText()); 
              if err then hiliteError(TagEdit.textbox, bad[1]) end;
            end);
                                           
  local revertButton = CreateFrame("button", nil, contentPane, "UIPanelButtonTemplate");
        revertButton:SetText(loc("TAG_EDIT_REVERT"));
        revertButton:SetWidth(90);
        revertButton:SetPoint("LEFT", testButton, "RIGHT", 7, 0)
        revertButton:SetScript("OnClick", function(self) TagEdit.textbox:SetText(Config.get(TagEdit.setting)); end);
  
  local defaultButton = CreateFrame("button", nil, contentPane, "UIPanelButtonTemplate");
        defaultButton:SetText(loc("TAG_EDIT_DEFAULT"));
        defaultButton:SetWidth(90);
        defaultButton:SetPoint("LEFT", revertButton, "RIGHT", 7, 0)
        defaultButton:SetScript("OnClick", function(self) TagEdit.textbox:SetText(Config.default(TagEdit.setting)); end);
  
  local cancelButton = CreateFrame("button", nil, contentPane, "UIPanelButtonTemplate");
        cancelButton:SetText(loc("TAG_EDIT_CANCEL"));
        cancelButton:SetWidth(90);
        cancelButton:SetPoint("LEFT", defaultButton, "RIGHT", 7, 0)
        cancelButton:SetScript("OnClick", function(self) TagEdit.setting = nil; TagEdit.draft = nil; TagEdit:Hide(); end);
  
  local testResultsLabel = contentPane:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        testResultsLabel:SetText(loc("TAG_EDIT_RESULTS"));
        testResultsLabel:SetPoint("TOP", cancelButton, "BOTTOM", 0, -5);
        testResultsLabel:SetPoint("LEFT", contentPane, "LEFT", 5, 0);
        testResultsLabel:SetTextColor(0.7, 0.7, 0.7)
        TagEdit.testResultsLabel = testResultsLabel;
  
  local testResults = contentPane:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        testResults:SetText("");
        testResults:SetPoint("TOPLEFT", testResultsLabel, "BOTTOMLEFT", 10, -5);
        testResults:SetPoint("RIGHT", contentPane, "RIGHT", -20, 0);
        testResults:SetHeight(40);
        testResults:SetJustifyH("LEFT");
        testResults:SetJustifyV("TOP");
        TagEdit.testResults = testResults;
  
  local function updateTagEdit()
        TagEdit.textbox:SetFocus()
        TagEdit.contentTitle:SetText(loc("CONFIG_" .. TagEdit.setting));
        TagEdit.displayBox:SetText(loc("CONFIG_" .. TagEdit.setting .. "_TT"));
           if   TagEdit.draft
           then TagEdit.textbox:SetText(TagEdit.draft)
                TagEdit.draft = nil;
           else TagEdit.textbox:SetText(Config.get(TagEdit.setting):gsub("%[p%]", "\n\n"):gsub("%[br%]", "\n"));
                TagEdit.testResults:SetText("");
                TagEdit.testResultsLabel:SetText(loc("TAG_EDIT_RESULTS"))
                TagEdit.testResultsLabel:SetTextColor(0.7, 0.7, 0.7)
           end;
  end; -- function
  
  RPTAGS.utils.rpuf.updateTagEdit = updateTagEdit;
  
  local function tagEdit(setting)
    TagEdit:Hide();
    if TagEdit.setting ~= setting then TagEdit.draft = nil; end;
    TagEdit.setting = setting;
    updateTagEdit();
    TagEdit:Show();
    end;
  
  RPTAGS.utils.rpuf.tagEdit = tagEdit;
  
  local function testTags(s)
    local err = false;
    local good = {};
    local bad = {};
    
    for tag in string.gmatch(s, "%[([^%]]+)%]")
        do if RPTAGS.utils.tags.exists(tag) 
              then table.insert(good, tag)
              else table.insert(bad, tag); err = true;
           end; --if
        end; -- do
    if err then TagEdit.testResultsLabel:SetText(loc("TAG_EDIT_RESULTS_FAIL"));
                TagEdit.testResultsLabel:SetTextColor(1, 0.1, 0.1);
                TagEdit.testResults:SetText(loc("TAG_TEST_FAIL" .. ("_SINGULAR" and err == 1 or "")) .. "|cffffffff" .. table.concat(bad, ", ") .. "|r")
           else TagEdit.testResultsLabel:SetText(loc("TAG_EDIT_RESULTS_PASS"));
                TagEdit.testResultsLabel:SetTextColor(0.1, 1, 0.1);
                TagEdit.testResults:SetText(loc("TAG_TEST_PASS"));
           end;
    return err, good, bad; 
  end;
  RPTAGS.utils.rpuf.testTags = testTags;
  
  local function fixTagErrors(s)
    local err, good, bad = testTags(s);
    if not err then return s end;
    for _, badTag in ipairs(bad)
    do s = s:gsub("%[" .. badTag .. "%]", "%[err%]" .. badTag .. "%[/err%]");
    end;
    return s;
  end -- function
  RPTAGS.utils.rpuf.fixTagErrors = fixTagErrors;

  -- Popup dialog
  
  StaticPopupDialogs["RPTAGS_TAGEDIT_ERROR"] = {
     showAlert    =  1,
     text         =  loc("TAG_EDIT_ERRORS"),
     button1      =  loc("TAG_EDIT_ERRORS_SAVE"),
     button2      =  loc("TAG_EDIT_ERRORS_EDIT"),
     exclusive    =  true,
     timeout      =  0, 
     whileDead    =  true,
     OnShow       =  function(self) TagEdit.textbox:ClearFocus(); end,
     OnAccept     =  function(self) 
                       RPTAGS.utils.config.set(TagEdit.setting, TagEdit.textbox:GetText()) 
                       TagEdit:Hide();
                     end,
     OnCancel     =  function(self) TagEdit.draft = TagEdit.textbox:GetText();
                       self:Hide();            
                       tagEdit(TagEdit.setting);
                     end,
  }; 

  return "utils-editor";

end); -- startup function
