-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
--
-- The licensor cannot revoke these freedoms as long as you follow the
-- license terms.
--
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------
--
-- utils-frames: Utilities for dealing with frames
-- full list of available functions at the end
--

local RPTAGS      = RPTAGS;

table.insert(RPTAGS.startup, function() -- startup function
  local fontFrame = CreateFrame('frame');
  local font = fontFrame:CreateFontString(nil, 'OVERLAY', 'GameFontNormal');
  local FONTFILE, FONTSIZE, _ = font:GetFont();

  local Const       = RPTAGS.const;
  local Utils       = RPTAGS.utils; if not Utils.rpuf then Utils.rpuf = {} end;
  local notify      = RPTAGS.utils.text.notify;
  local Ora         = RPTAGS.utils.text.ora;

  local Config      = RPTAGS.utils.config;
  local loc         = RPTAGS.utils.locale.loc;
  local PANELS      = { "NamePanel", "InfoPanel", "Portrait", "DetailsPanel", "StatusBarPanel", "Icon_1Panel", "Icon_2Panel", "Icon_3Panel", 
                        "Icon_4Panel", "Icon_5Panel", "Icon_6Panel", };
  local FRAME_LIST  = { "RPUF_Player", "RPUF_Target", "RPUF_TargetTarget", "RPUF_Focus" };

  local function FRAMES()
     local frames = {};
     for _, f in ipairs(FRAME_LIST) do table.insert(frames, _G[f]) end;
     return frames;
  end; -- function

  local function GAP(n)  if not n then n = 1 end; return Config.get("GAPSIZE") * n;      end;
  local function NGAP(n) if not n then n = 1 end; return Config.get("GAPSIZE") * n * -1; end;

  local function frameLayout(frame)
    local  frameName = frame:GetName();
    if     frameName == "RPUF_Player"       then return Config.get("PLAYERLAYOUT")
    elseif frameName == "RPUF_Target"       then return Config.get("TARGETLAYOUT")
    elseif frameName == "RPUF_Focus"        then return Config.get("FOCUSLAYOUT")
    elseif frameName == "RPUF_TargetTarget" then return Config.get("FOCUSLAYOUT")
    else   return "COMPACT"
    end;
  end; -- function

  local function elementHeight(element, layout)
    local height;

    if     not element                                          then height = 0
    elseif element == 'NamePanel'                                    then height = FONTSIZE + 4;

    elseif element == 'InfoPanel'                                    then height = FONTSIZE + 2;

    elseif element == 'Portrait'  and layout  == "PAPERDOLL"    then height = Config.get("PORTWIDTH") * 3;
    elseif element == 'Portrait'  and layout  == "THUMBNAIL"    then height = Config.get("PORTWIDTH") * 1.5;
    elseif element == 'Portrait'                                then height = Config.get("PORTWIDTH") * 1.5;

    elseif element == 'DetailsPanel'                                 then height = Config.get("DETAILHEIGHT");

    elseif element == 'StatusBarPanel'                               then height = Config.get("STATUSHEIGHT");

    elseif element == 'Icon_1Panel'     and layout  == "PAPERDOLL"    then height = Config.get("ICONWIDTH") * 2;
    elseif element == 'Icon_1Panel'                                   then height = Config.get("ICONWIDTH");
    elseif element == 'Icon_2Panel'     and layout  == "PAPERDOLL"    then height = Config.get("ICONWIDTH") * 2;
    elseif element == 'Icon_2Panel'                                   then height = Config.get("ICONWIDTH");
    elseif element == 'Icon_3Panel'     and layout  == "PAPERDOLL"    then height = Config.get("ICONWIDTH") * 2;
    elseif element == 'Icon_3Panel'                                   then height = Config.get("ICONWIDTH");
    elseif element == 'Icon_4Panel'     and layout  == "PAPERDOLL"    then height = Config.get("ICONWIDTH") * 2;
    elseif element == 'Icon_4Panel'                                   then height = Config.get("ICONWIDTH");
    elseif element == 'Icon_5Panel'     and layout  == "PAPERDOLL"    then height = Config.get("ICONWIDTH") * 2;
    elseif element == 'Icon_5Panel'                                   then height = Config.get("ICONWIDTH");
    elseif element == 'Icon_6Panel'     and layout  == "PAPERDOLL"    then height = Config.get("ICONWIDTH") * 2;
    elseif element == 'Icon_6Panel'                                   then height = Config.get("ICONWIDTH");
                                                                else height = 0;
    end;

    return height;
  end;

  local function elementWidth(element, layout)
    local width;

    if not element then width = 0
    elseif element == 'NamePanel'       and layout  == "THUMBNAIL"  then width = Config.get("PORTWIDTH") - GAP(1);
    elseif element == 'NamePanel'       and layout  == 'PAPERDOLL'  then width = Config.get("PORTWIDTH") * 2;
    elseif element == 'NamePanel'                                   then width = Config.get("INFOWIDTH");
    
    elseif element == 'InfoPanel'       and layout  == 'PAPERDOLL'  then width = Config.get("PORTWIDTH") * 2;
    elseif element == 'InfoPanel'                                   then width = Config.get("INFOWIDTH");

    elseif element == 'Portrait'   and layout  == "PAPERDOLL"  then width = Config.get("PORTWIDTH") * 2;
    elseif element == 'Portrait'   and layout  == "THUMBNAIL"  then width = Config.get("PORTWIDTH");
    elseif element == 'Portrait'                               then width = Config.get("PORTWIDTH");

    elseif element == 'DetailsPanel'                                then width = Config.get("INFOWIDTH");
    elseif element == 'StatusBarPanel'  and layout  == "FULL"       then width = Config.get("PORTWIDTH")
                                                                            + GAP(1) + Config.get("INFOWIDTH")
                                                                            + GAP(1) + Config.get("ICONWIDTH");
    elseif element == 'StatusBarPanel'  and layout  == "ABRIDGED"   then width = math.max(
                                                                            -- formula #1
                                                                            Config.get("ICONWIDTH")
                                                                            + GAP(0.75) + Config.get("INFOWIDTH"),
                                                                            -- formula #2
                                                                            Config.get("ICONWIDTH")
                                                                            + GAP(0.75) + Config.get("ICONWIDTH")
                                                                            + GAP(0.75) + Config.get("ICONWIDTH")
                                                                            + GAP(0.75) + Config.get("ICONWIDTH")
                                                                            + GAP(0.75) + Config.get("ICONWIDTH")
                                                                           );
    elseif element == 'StatusBarPanel'  and layout  == "COMPACT"    then width = 0;

    elseif element == 'Icon_1Panel'      and layout  == "PAPERDOLL"  then width = Config.get("ICONWIDTH") * 2;
    elseif element == 'Icon_2Panel'      and layout  == "PAPERDOLL"  then width = Config.get("ICONWIDTH") * 2;
    elseif element == 'Icon_3Panel'      and layout  == "PAPERDOLL"  then width = Config.get("ICONWIDTH") * 2;
    elseif element == 'Icon_4Panel'      and layout  == "PAPERDOLL"  then width = Config.get("ICONWIDTH") * 2;
    elseif element == 'Icon_5Panel'      and layout  == "PAPERDOLL"  then width = Config.get("ICONWIDTH") * 2;
    elseif element == 'Icon_6Panel'      and layout  == "PAPERDOLL"  then width = Config.get("ICONWIDTH") * 2;
    elseif element == 'Icon_1Panel'                                  then width = Config.get("ICONWIDTH");
    elseif element == 'Icon_2Panel'                                  then width = Config.get("ICONWIDTH");
    elseif element == 'Icon_3Panel'                                  then width = Config.get("ICONWIDTH");
    elseif element == 'Icon_4Panel'                                  then width = Config.get("ICONWIDTH");
    elseif element == 'Icon_5Panel'                                  then width = Config.get("ICONWIDTH");
    elseif element == 'Icon_6Panel'                                  then width = Config.get("ICONWIDTH");
    
                                                               else width = 0;
    end;

    return width;
    end;

  local function frameDimensions(layout)
    local border, width, height;

    if   Config.get("RPUF_BACKDROP") == "BLIZZTOOLTIP"
    then border = 5
    else border = 0 end;

    if     layout == "FULL" -- -----------------------------------------------------------------------------------------------------------
    then   width  = border
                    + GAP(1) + elementWidth('Icon_1Panel', layout)
                    + GAP(1) + elementWidth('InfoPanel', layout)
                    + GAP(1) + elementWidth('Portrait', layout)
                    + GAP(1) + border;

           height = math.max(
                       -- formula #1
                       border
                       + GAP(1) + elementHeight('Icon_1Panel', layout)
                       + GAP(1) + elementHeight('Icon_2Panel', layout)
                       + GAP(1) + elementHeight('Icon_3Panel', layout)
                       + GAP(1) + elementHeight('Icon_4Panel', layout)
                       + GAP(1) + elementHeight('Icon_5Panel', layout)
                       + GAP(1) + elementHeight('Icon_6Panel', layout)
                       + GAP(1) + elementHeight('StatusBarPanel', "FULL")
                       + GAP(1) + border,
                       -- formula #2
                       border
                       + GAP(1) + elementHeight('NamePanel', layout)
                       + GAP(1) + elementHeight('DetailsPanel', layout)
                       + GAP(1) + elementHeight('StatusBarPanel', "FULL")
                       + GAP(1) + elementHeight('InfoPanel', layout)
                       + GAP(1) + border,
                       -- formula #3
                       border
                       + GAP(1) + elementHeight('Portrait', layout)
                       + GAP(1) + elementHeight('StatusBarPanel', "FULL")
                       + GAP(1) + border
                       );

    elseif layout == "THUMBNAIL" -- -------------------------------------------------------------------------------------------------------------
    then   width  =  border + elementWidth('Portrait', layout)  + border;
           height =  border + elementHeight('Portrait', layout) + border;

    elseif layout == "PAPERDOLL" -- -------------------------------------------------------------------------------------------------------------
    then   width  =  math.max(
                     -- formula #1
                     border
                     + GAP(1) + elementWidth('Icon_1Panel', layout)
                     + GAP(1) + elementWidth('InfoPanel', layout)
                     + GAP(1) + elementWidth('Icon_4Panel', layout)
                     + GAP(1) + border,
                     -- formula #2
                     border
                     + GAP(1) + elementWidth('Icon_1Panel', layout)
                     + GAP(1) + elementWidth('Portrait', layout)
                     + GAP(1) + elementWidth('Icon_4Panel', layout)
                     + GAP(1) + border
                     );
           height =  math.max(
                     -- formula #1
                     border
                     + GAP(1) + elementHeight('NamePanel', layout)
                     + GAP(1) + elementHeight('InfoPanel', layout)
                     + GAP(1) + elementHeight('Portrait', layout)
                     + border, -- no gap needed here
                     -- formula #2
                     border
                     + GAP(1) + elementHeight('NamePanel', layout)
                     + GAP(1) + elementHeight('InfoPanel', layout)
                     + GAP(4) + elementHeight('Icon_1Panel', layout)
                     + GAP(4) + elementHeight('Icon_2Panel', layout)
                     + GAP(4) + elementHeight('Icon_3Panel', layout)
                     + GAP(1) + border
                    );

    elseif layout == "ABRIDGED" -- -----------------------------------------------------------------------------------------------------------
    then   width = math.max(
                     -- formula #1
                     border
                     + GAP(1) + elementWidth('Icon_1Panel', layout)
                     + GAP(1) + elementWidth('NamePanel', layout)
                     + GAP(1) + border,
                     -- formula #2
                     border
                     + GAP(1) + elementWidth('Icon_1Panel', layout)
                     + GAP(1) + elementWidth('InfoPanel', layout)
                     + GAP(1) + border,
                     -- formula #3
                     border
                     + GAP(1) + elementWidth('StatusBarPanel', "ABRIDGED")
                     + GAP(1) + border,
                     -- formula #4
                     border
                     + GAP(1) + elementWidth('Icon_2Panel', layout)
                     + GAP(1) + elementWidth('Icon_3Panel', layout)
                     + GAP(1) + elementWidth('Icon_4Panel', layout)
                     + GAP(1) + elementWidth('Icon_5Panel', layout)
                     + GAP(1) + elementWidth('Icon_6Panel', layout)
                     + GAP(1) + border
                     );
            height = math.max(
                       -- formula #1
                       border
                       + GAP(1) + elementHeight('Icon_1Panel', layout)
                       + GAP(1) + elementHeight('StatusBarPanel', "ABRIDGED")
                       + GAP(1) + elementHeight('Icon_2Panel', layout)
                       + GAP(1) + border,
                       -- formula #2
                       border
                       + GAP(1) + elementHeight('NamePanel' , layout)
                       + GAP(1) + elementHeight('InfoPanel', layout)
                       + GAP(1) + elementHeight('StatusBarPanel', "ABRIDGED")
                       + GAP(1) + elementHeight('Icon_2Panel', layout)
                       + GAP(1) + border
                       ); --

    elseif layout == "COMPACT" -- -----------------------------------------------------------------------------------------------------------
    then   width = border
                   + GAP(0.5) + elementWidth('Icon_1Panel', layout)
                   + GAP(0.5) + elementWidth('NamePanel', layout)
                   + GAP(0.5) + border;
           height = math.max(
                      -- formula #1
                      border
                      + GAP(0.5) + elementHeight('NamePanel', layout)
                      + GAP(0.5) + elementHeight('InfoPanel', layout)
                      + GAP(0.5) + border,
                      -- formula #2
                      border
                      + GAP(0.5) + elementHeight('Icon_1Panel', layout)
                      + GAP(0.5) + border
                    );

    elseif layout == "HIDDEN" -- ----------------------------------------------------------------------------------------------------------
    then   width, height = 0, 0;
    end; -- if layout ---------------------------------------------------------------------------------------------------------------------

    return width, height;
  end;

  local function elementLeftTop(element, layout)
    local left, top, border, nborder;

    if   Config.get("RPUF_BACKDROP") == "BLIZZTOOLTIP" then border = 5; nborder = -5;
    else                                                    border = 0; nborder =  0; 
    end; --if

    if   layout == "FULL"  -- -----------------------------------------------------------------------------------------------------------
    then local iconLeft = border + GAP(1);
         if not element
         then   left    =  border + GAP(1);
                top     =  nborder + NGAP(1);
         elseif element == 'Portrait'
         then   left    =  border
                           + GAP(1) + elementWidth('Icon_1Panel', layout)
                           + GAP(1);
                top     =  nborder + NGAP(1);
         elseif element == 'Icon_1Panel'
         then   left    =  iconLeft;
                top     =  nborder + NGAP(1);
         elseif element == 'Icon_2Panel'
         then   left    =  iconLeft;
                top     =  nborder
                           + NGAP(1) - elementHeight('Icon_1Panel', layout)
                           + NGAP(1);
         elseif element == 'Icon_3Panel'
         then   left    =  iconLeft;
                top     =  nborder
                           + NGAP(1) - elementHeight('Icon_1Panel', layout)
                           + NGAP(1) - elementHeight('Icon_2Panel', layout)
                           + NGAP(1);
         elseif element == 'Icon_4Panel'
         then   left    =  iconLeft;
                top     =  nborder
                           + NGAP(1) - elementHeight('Icon_1Panel', layout)
                           + NGAP(1) - elementHeight('Icon_2Panel', layout)
                           + NGAP(1) - elementHeight('Icon_3Panel', layout)
                           + NGAP(1);
         elseif element == 'Icon_5Panel'
         then   left    =  iconLeft;
                top     =  nborder
                           + NGAP(1) - elementHeight('Icon_1Panel', layout)
                           + NGAP(1) - elementHeight('Icon_2Panel', layout)
                           + NGAP(1) - elementHeight('Icon_3Panel', layout)
                           + NGAP(1) - elementHeight('Icon_4Panel', layout)
                           + NGAP(1);
         elseif element == 'Icon_6Panel'
         then   left    =  iconLeft;
                top     =  nborder
                           + NGAP(1) - elementHeight('Icon_1Panel', layout)
                           + NGAP(1) - elementHeight('Icon_2Panel', layout)
                           + NGAP(1) - elementHeight('Icon_3Panel', layout)
                           + NGAP(1) - elementHeight('Icon_4Panel', layout)
                           + NGAP(1) - elementHeight('Icon_5Panel', layout)
                           + NGAP(1);
         elseif element == 'NamePanel'
         then   left    =  border
                           + GAP(1) + elementWidth('Icon_1Panel', layout)
                           + GAP(1) + elementWidth('Portrait', layout)
                           + GAP(1);
                top     =  nborder
                           + NGAP(1);
         elseif element == 'InfoPanel'
         then   left    =  border
                           + GAP(1) + elementWidth('Icon_1Panel', layout)
                           + GAP(1) + elementWidth('Portrait', layout)
                           + GAP(1);
                top     =  nborder
                           + NGAP(1) - elementHeight('NamePanel', layout)
                           + NGAP(1);
         elseif element == 'DetailsPanel'
         then   left    =  border
                           + GAP(1) + elementWidth('Icon_1Panel', layout)
                           + GAP(1) + elementWidth('Portrait', layout)
                           + GAP(1);
                top     =  nborder
                           + NGAP(1) - elementHeight('NamePanel', layout)
                           + NGAP(1) - elementHeight('InfoPanel', layout)
                           + NGAP(1);
         elseif element == 'StatusBarPanel'
         then   left    =  border + GAP(1);
                top     =  math.min(
                             -- formula #1
                             nborder
                             + NGAP(1) - elementHeight('Icon_1Panel', layout)
                             + NGAP(1) - elementHeight('Icon_2Panel', layout)
                             + NGAP(1) - elementHeight('Icon_3Panel', layout)
                             + NGAP(1) - elementHeight('Icon_4Panel', layout)
                             + NGAP(1) - elementHeight('Icon_5Panel', layout)
                             + NGAP(1) - elementHeight('Icon_6Panel', layout)
                             + NGAP(1),
                             -- formula #2
                             nborder
                             + NGAP(1) - elementHeight('Portrait', layout)
                             + NGAP(1),
                             -- formula #3
                             nborder
                             + NGAP(1) - elementHeight('NamePanel', layout)
                             + NGAP(1) - elementHeight('InfoPanel', layout)
                             + NGAP(1) - elementHeight('DetailsPanel', layout)
                             + NGAP(1));
         else   left  = border + GAP(1);
                top   = nborder + NGAP(1);
         end; -- if element

       elseif layout == "THUMBNAIL" -- ----------------------------------------------------------------------------------------------------------
       then if not element               then left = border;            top = nborder;
            elseif element == 'Portrait' then left = border;            top = nborder
            elseif element == 'NamePanel'     then left = border + GAP(0.5); top = nborder - elementHeight('Portrait', layout) + elementHeight('NamePanel', layout);
            else                              left = 5000;              top = 5000;
            end -- if element

       elseif layout == "PAPERDOLL" -- ----------------------------------------------------------------------------------------------------------
       then local leftIconLeft  = border + GAP(1);
            local rightIconLeft = math.max(
                                  -- formula #1
                                  border
                                  + GAP(1) + elementWidth('Icon_1Panel', layout)
                                  + GAP(1) + elementWidth('Portrait', layout)
                                  + GAP(1),
                                  -- formula #2
                                  border
                                  + GAP(1) + elementWidth('Icon_1Panel', layout)
                                  + GAP(1) + elementWidth('InfoPanel', layout)
                                  + GAP(1)
                                );
            if not element
            then   left    = border + GAP(1);
                   top     = nborder + NGAP(1);
            elseif element == 'NamePanel'
            then   left    = border
                             + GAP(1) + elementWidth('Icon_1Panel', layout)
                             + GAP(1);
                   top     = nborder + NGAP(1);
            elseif element == 'InfoPanel'
            then   left    = border
                             + GAP(1) + elementWidth('Icon_1Panel', layout)
                             + GAP(1)
                   top     = nborder
                             + NGAP(1) - elementHeight('NamePanel', layout)
                             + NGAP(1);
            elseif element == 'Portrait'
            then   left    = border
                             + GAP(1) + elementWidth('Icon_1Panel', layout)
                             + GAP(1);
                   top     = nborder
                             + NGAP(1) - elementHeight('NamePanel', layout)
                             + NGAP(1) - elementHeight('InfoPanel', layout)
                             + NGAP(1);
            elseif element == 'Icon_1Panel'
            then   left    = leftIconLeft;
                   top     = nborder
                             + NGAP(1) - elementHeight('NamePanel', layout)
                             + NGAP(1) - elementHeight('InfoPanel', layout)
                             + NGAP(4);
            elseif element == 'Icon_2Panel'
            then   left    = leftIconLeft;
                   top     = nborder
                             + NGAP(1) - elementHeight('NamePanel', layout)
                             + NGAP(1) - elementHeight('InfoPanel', layout)
                             + NGAP(4) - elementHeight('Icon_1Panel', layout)
                             + NGAP(4);
            elseif element == 'Icon_3Panel'
            then   left    = leftIconLeft;
                   top     = nborder
                             + NGAP(1) - elementHeight('NamePanel', layout)
                             + NGAP(1) - elementHeight('InfoPanel', layout)
                             + NGAP(4) - elementHeight('Icon_1Panel', layout)
                             + NGAP(4) - elementHeight('Icon_2Panel', layout)
                             + NGAP(4);
             elseif element == 'Icon_4Panel'
             then   left    = rightIconLeft;
                    top     = nborder
                              + NGAP(1) - elementHeight('NamePanel', layout)
                              + NGAP(1) - elementHeight('InfoPanel', layout)
                              + NGAP(4);
             elseif element == 'Icon_5Panel'
             then   left    = rightIconLeft;
                    top     = nborder
                              + NGAP(1) - elementHeight('NamePanel', layout)
                              + NGAP(1) - elementHeight('InfoPanel', layout)
                              + NGAP(4) - elementHeight('Icon_4Panel', layout)
                              + NGAP(4);
             elseif element == 'Icon_6Panel'
             then   left    = rightIconLeft;
                    top     = nborder
                              + NGAP(1) - elementHeight('NamePanel', layout)
                              + NGAP(1) - elementHeight('InfoPanel', layout)
                              + NGAP(4) - elementHeight('Icon_4Panel', layout)
                              + NGAP(4) - elementHeight('Icon_5Panel', layout)
                              + NGAP(4);
             else   left, top = 5000, 5000;
             end; -- if element
       elseif layout == "ABRIDGED" -- -----------------------------------------------------------------------------------------------------------
       then local iconTop = math.min(
                              -- formula #1
                              nborder
                              + NGAP(1) - elementHeight('Icon_1Panel', layout)
                              + NGAP(1) - elementHeight('StatusBarPanel', "ABRIDGED")
                              + NGAP(1),
                              -- formula #2
                              nborder
                              + NGAP(1) - elementHeight('NamePanel', layout)
                              + NGAP(1) - elementHeight('InfoPanel', layout)
                              + NGAP(1) - elementHeight('StatusBarPanel', "ABRIDGED")
                              + NGAP(1)
                            );

            if not element
            then   left    =  border + GAP(1)
                   top     =  nborder + NGAP(1);
            elseif element == 'NamePanel'
            then   left    =  border
                              + GAP(1) + elementWidth('Icon_1Panel', layout)
                              + GAP(1)
                   top     =  nborder
                              + NGAP(1);
            elseif element == 'InfoPanel'
            then   left    =  border
                              + GAP(1) + elementWidth('Icon_1Panel', layout)
                              + GAP(1);
                   top     =  nborder
                              + NGAP(1) - elementHeight('NamePanel', layout)
                              + NGAP(1);
            elseif element == 'Icon_1Panel'
            then   left    =  border + GAP(1);
                   top     =  nborder + NGAP(1);
            elseif element == 'StatusBarPanel'
            then   left    =  border + GAP(1);
                   top     =  math.min(
                                -- formula #1
                                nborder
                                + NGAP(1) - elementHeight('Icon_1Panel', layout)
                                + NGAP(1),
                                -- formula #2
                                nborder
                                + NGAP(1) - elementHeight('NamePanel' , layout)
                                + NGAP(1) - elementHeight('InfoPanel', layout)
                                + NGAP(1)
                              );
            elseif element == 'Icon_2Panel'
            then   left    =  border + GAP(1)
                   top     =  iconTop;
            elseif element == 'Icon_3Panel'
            then   left    =  border
                              + GAP(1) + elementHeight('Icon_2Panel', layout)
                              + GAP(1);
                   top     =  iconTop;
            elseif element == 'Icon_4Panel'
            then   left    =  border
                              + GAP(1) + elementHeight('Icon_2Panel', layout)
                              + GAP(1) + elementHeight('Icon_3Panel', layout)
                              + GAP(1);
                   top     =  iconTop;
            elseif element == 'Icon_5Panel'
            then   left    =  border
                              + GAP(1) + elementHeight('Icon_2Panel', layout)
                              + GAP(1) + elementHeight('Icon_3Panel', layout)
                              + GAP(1) + elementHeight('Icon_4Panel', layout)
                              + GAP(1);
                   top     =  iconTop;
            elseif element == 'Icon_6Panel'
            then   left    =  border
                              + GAP(1) + elementHeight('Icon_2Panel', layout)
                              + GAP(1) + elementHeight('Icon_3Panel', layout)
                              + GAP(1) + elementHeight('Icon_4Panel', layout)
                              + GAP(1) + elementHeight('Icon_5Panel', layout)
                              + GAP(1);
                   top     =  iconTop;
            else   left, top = 5000, 5000;
            end; -- if element

       elseif layout == "COMPACT" -- -----------------------------------------------------------------------------------------------------------
       then
            if not element
            then   left    =  border  + GAP(0.5);
                   top     =  nborder + NGAP(0.5);
            elseif element == 'Icon_1Panel'
            then   left    =  border  + GAP(0.5);
                   top     =  nborder + NGAP(0.5);
            elseif element == 'NamePanel'
            then   left    =  border
                              + GAP(0.5) + elementWidth('Icon_1Panel', layout)
                              + GAP(0.5);
                   top     =  nborder
                              + NGAP(0.5);
            elseif element == 'InfoPanel'
            then   left    =  border
                              + GAP(0.5) + elementWidth('Icon_1Panel', layout)
                              + GAP(0.5);
                   top     =  nborder
                              + NGAP(1) - elementHeight('NamePanel', layout);
            else   left, top = 5000, 5000
            end; -- if element
       else left, top = 5000, 5000
       end; -- if layout

    return left, top;
  end;

  local function elementTop(element, layout)  local _,   top = elementLeftTop(element, layout) return top;  end;
  local function elementLeft(element, layout) local left, _  = elementLeftTop(element, layout) return left; end;

  -- local function debugElements(unit, element)
  --   local layout = frameLayout(unit);
  --   if not element
  --   then local width, height = frameDimensions(layout);
  --        Ora("u = ", unit, ", layout = ", layout, ", w = ", width, ", h = ", height);
  --   else local width  = elementWidth(element);
  --        local height = elementHeight(element);
  --        local left   = elementLeft(unit, element);
  --        local top    = elementTop(unit, element);
  --        Ora("u = ", unit, ", e = ", element, ", w = ", width, ", h = ", height, ", l = ", left, ", t = ", top);
  --   end; --if
  -- end; -- function

  local function refreshTags(frame)
    if   frame.__tags 
    then for _, element in ipairs(frame.__tags) 
         do  element:UpdateTag(); 
         end;  -- for
    end; -- if
  end; -- function

  local function refreshAllTags() for _, frame in ipairs(FRAMES()) do refreshTags(frame) end; end;

  local function setTag(frame, element, value, field)
    frame:Tag(frame[element][field or "text"], RPTAGS.utils.rpuf.fixTagErrors(value)); 
    frame:UpdateTags();
  end; -- function

  local function tagAll(element, value, field) for _, frame in ipairs(FRAMES()) do setTag(frame, element, value, field) end; end;

  local function resizePanel(frame, panel, layout)
    local w    = elementWidth( panel, layout); -- calculate the width
    local h    = elementHeight(panel, layout); -- calculate the height
    local top  = elementTop(   panel, layout); -- calculate the point
    local left = elementLeft(  panel, layout);

    if panel:match("^Icon_") then h = h + 3; w = w * 2; frame[panel].text:SetFont(FONTFILE, h - 1); end;

    frame[panel]:SetHeight(h);
    frame[panel]:SetWidth(w);
    frame[panel]:SetPoint('TOPLEFT', frame, 'TOPLEFT', left, top);
    if frame[panel].text 
       then frame[panel].text:SetHeight(h);
            frame[panel].text:SetWidth(w);
       end;
  end; -- function

  local function resizeFrame(frame)
    local layout = frameLayout(frame);
    for _, panel in ipairs(PANELS) do resizePanel(frame, panel, layout); end;
    local w, h = frameDimensions(layout);
    frame:SetWidth(w);
    frame:SetHeight(h);
    -- frame.Portrait:SetCamera(portraitType(layout));
  end; -- function

  local function resizeAllFrames() for _, frame in ipairs(FRAMES()) do resizeFrame(frame) end; end;

  local function setAllBackground(alpha, colorRPUF)
    local bgRed, bgGreen, bgBlue = RPTAGS.utils.color.hexaToNumber(colorRPUF);
    for _, frame in ipairs(FRAMES())
    do frame:SetBackdropColor(bgRed / 255, bgGreen / 255, bgBlue / 255, alpha/100);
    end;
    RPUF_Tooltip:UpdateColors();
  end; -- function

  local function setAllFrameBackdrop(backdrop)
    if not Const.BACKDROP[backdrop] then return false end;
    local bgRed, bgGreen, bgBlue = RPTAGS.utils.color.hexaToNumber(Config.get("COLOR_RPUF"));
    local alpha = Config.get("RPUFALPHA");
    for _, frame in ipairs(FRAMES())
    do frame:SetBackdrop(Const.BACKDROP[backdrop])
       frame:SetBackdropColor(bgRed / 255, bgGreen / 255, bgBlue / 255, alpha/100);
       resizeFrame(frame)
    end; -- for
  end; -- function

  local function lockFrames() -- -------------------------------------------------------------------------------------------------------------
    for _, frame in ipairs(FRAMES())
    do  if Config.get("LOCK_FRAMES") then frame.DragFrame:Hide() else frame.DragFrame:Show() end;
    end; -- do
    if   Config.get("LOCK_FRAMES") then notify("LOCKING_FRAMES") else notify("UNLOCKING_FRAMES") end; -- if
  end; -- function

  local function resetAllLocations()
    local function lookup(unit) return _G[unit] end;

    for _, frame in ipairs(FRAMES())
    do local layout = frameLayout(frame);
       local w, h   = frameDimensions(layout);
       local IP = Const.RPUF.INITIAL_POSITION[frame.unit];

       frame:SetWidth(w);
       frame:SetHeight(h);
       frame:ClearAllPoints();
       frame:SetPoint(IP.pt, lookup(IP.relto), IP.relpt, IP.x, IP.y);
       RP_TagsDB[frame.unit .. "UFlocation"] = nil;
    end; -- do
  end; -- function

  local function setHide(init)
    local hide = Config.get("RPUF_HIDE");
    for _, frame in ipairs(FRAMES())
    do UnregisterStateDriver(frame, 'visibility');
       local vis = "[@player,dead] hide; [@player,vehicleui] hide; [@player,petbattle] hide; [@" .. frame.unit .. ",exists] show; hide";
       if   hide then vis = "[@player,combat] hide; " .. vis end;

       RegisterStateDriver(frame, 'visibility', vis)
    end; -- for

    if hide and not init then notify("RPUF_HIDE_IN_COMBAT") elseif not init then notify("RPUF_SHOW_IN_COMBAT") end; -- if not init

  end; -- function

  local function refreshAll()
    resizeAllFrames();
    refreshAllTags();
--     setAllPortraitType();
  end; -- function
    
  -- functions available through RPTAGS.utils.rpuf
  if not Utils.rpuf.frame   then Utils.rpuf.frame   = {} end;
  if not Utils.rpuf.element then Utils.rpuf.element = {} end;

  Utils.rpuf.debug               = debugElements;
  Utils.rpuf.frame.refresh       = refreshTags;
  Utils.rpuf.frame.resize        = resizeFrame;
  Utils.rpuf.frame.size          = frameDimensions;
  Utils.rpuf.frame.layout        = frameLayout;
  Utils.rpuf.frame.tag           = setTag;
  Utils.rpuf.frame.tagall        = tagAll;
  Utils.rpuf.frame.setbg         = setAllBackground;
  Utils.rpuf.frame.moveall       = setAllMovable;
  Utils.rpuf.frame.lockall       = lockFrames;
  Utils.rpuf.frame.resetloc      = resetAllLocations;
  Utils.rpuf.frame.backdrop      = setAllFrameBackdrop;
  Utils.rpuf.frame.resizeall     = refreshAll;
  Utils.rpuf.frame.refreshall    = refreshAll;
  Utils.rpuf.frame.sethide       = setHide;

  Utils.rpuf.element.resize      = resizePanel;
  Utils.rpuf.element.height      = elementHeight;
  Utils.rpuf.element.width       = elementWidth;
  Utils.rpuf.element.topleft     = elementLeftTop;
  Utils.rpuf.element.left        = elementLeft;
  Utils.rpuf.element.top         = elementTop;

  return "utils-frames"
end); -- startup function
