-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
--
-- The licensor cannot revoke these freedoms as long as you follow the
-- license terms.
--
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Utils-Config: Utilities for reading and registering configuration values

local RPTAGS     = RPTAGS;

table.insert(RPTAGS.startup, function() -- startup function

  if   RPTAGS.msp == "trp3"
  then
    local Const      = RPTAGS.const;
    local Utils      = RPTAGS.utils;
  
    if not RPTAGS.cache then RPTAGS.cache = {} end;
    if not Utils.config then Utils.config = {} end;
    if not Utils.help   then Utils.help = {} end;
    local Config = Utils.config;
  
    local Pages = Const.CONFIG.PAGES;
  
    local loc         = Utils.locale.loc;
    local hi          = Utils.text.hilite;
    local Format      = Utils.format;                 -- the subcategory
    local Ora         = Utils.text.ora;
  
    local Keys        = Const.CONFIG.KEYS;
    local Callback    = Const.CONFIG.CALLBACKS;
    local H1          = "TRP3_ConfigH1";
    local CHECK       = "TRP3_ConfigCheck";
    local SELECT      = "TRP3_ConfigDropDown";
    local COLOR       = "TRP3_ConfigColorPicker";
    local BUTTON      = "TRP3_ConfigButton";
    local SLIDER      = "TRP3_ConfigSlider";
    local BOX         = "TRP3_ConfigEditBox";
    local DEPS        = "dependentOnOptions";
    local WIDGET      = "TRP3_ConfigurationWidget";
  
    local LIST        = Const.CONFIG.LIST;
  
    -- these helper functions define various types of trp3 help screen widgets
    local function h1(key, currentPage)
      Pages[key] = currentPage;
      return {
        inherit     = H1,
        title       = loc("CONFIG_H1_" .. key) };
      end; -- h1()
  
    local function check(key, dep, currentPage)
      Pages[key] = currentPage;
      return {
        inherit     = CHECK,
        title       = hi(loc("CONFIG_" .. key)),
        widgetName  = WIDGET .. Keys[key],
        configKey   = Keys[key],
        help        = hi(loc("CONFIG_" .. key .. "_TT")),
        [DEPS]      = dep };
      end; -- check()
  
    local function list(key, dep, currentPage)
      Pages[key] = currentPage;
      local help = hi(loc("CONFIG_" .. key .. "_TT"));
      if string.len(help) == 0 then help = nil; end;
      return {
        inherit     = SELECT,
        title       = hi(loc("CONFIG_" .. key)),
        configKey   = Keys[key],
        widgetName  = WIDGET .. Keys[key],
        listContent = LIST[key],
        help        = help,
        [DEPS]      = dep };
      end; -- list()
  
    local function slider(key, dep, min, max, step, currentPage)
      Pages[key] = currentPage;
      local help = hi(loc("CONFIG_" .. key .. "_TT"));
      if string.len(help) == 0 then help = nil; end;
      return {
          inherit   = SLIDER,
          title     = hi(loc("CONFIG_" .. key)),
          configKey = Keys[key],
          widgetName = WIDGET .. Keys[key],
          min       = min or 0,
          max       = max or 60,
          step      = step or 5,
          integer   = true,
          help      = help,
          [DEPS]    = dep };
       end; -- slider()
  
    local function color(key, dep, currentPage)
      Pages[key] = currentPage;
      key = "COLOR_" .. key;
      return {
        inherit     = COLOR,
        title       = hi(loc("CONFIG_" .. key)),
        configKey   = Keys[key],
        widgetName = WIDGET .. Keys[key],
        help        = hi(loc("CONFIG_" .. key .. "_TT")),
        [DEPS]      = dep };
      end; -- color()
  
    local function button(key, text, deps, currentPage)
      Pages[key] = currentPage;
      local title   = hi(loc("CONFIG_BTN_" .. key));
      text          = hi(loc(text or "TOGGLE"));
      return {
        inherit     = BUTTON,
        title       = title,
        configKey   = Keys[key],
        help        = hi(loc("CONFIG_BTN_" .. key .. "_TT")),
          widgetName = WIDGET .. Keys[key],
        [DEPS]      = deps,
        text        = text,
        callback    = Callback[key] or function() Ora("Beep!") end,
        };
      end; -- button()
  
    local function tagedit(key, label, deps, currentPage)
      Pages[key] = currentPage;
      return {
        inherit     = BUTTON,
        title       = hi(loc("CONFIG_" .. key)),
        text        = hi(loc(label or "EDIT_TAGS")),
        configKey   = Keys[key],
        widgetName  = WIDGET .. Keys[key],
        help        = hi(loc("CONFIG_" .. key .. "_TT")),
        [DEPS]      = deps,
        callback    = function() Utils.rpuf.tagEdit(key); end,
        };
    end; -- tagedit()
  
    local function link(key, text, instr, currentPage)
      Pages[key] = currentPage;
      local title   = hi(loc("CONFIG_BTN_" .. key));
      local url     = loc("CONFIG_BTN_" .. key .. "_URL");
      text          = hi(loc(text  or "GET_LINK"));
      instr         = hi(loc(instr or "GET_LINK_INSTRUCTIONS"));
  
      return {
        inherit     = BUTTON,
        title       = title,
        configKey   = Keys[key],
        widgetName  = WIDGET .. (Keys[key] or "rptags_" .. key),
        help        = hi(loc("CONFIG_BTN_" .. key .. "_TT")),
        [DEPS]      = nil,
        text        = text,
        callback    = Callback[key] or function() Format.popup:OpenURL(url, title, instr); end -- end of callback function
        };
      end; -- link()
  
    local function textbox(key, dep, currentPage)
      Pages[key] = currentPage;
     return {
        inherit   = BOX,
        title     = hi(loc("CONFIG_" .. key)),
          widgetName = WIDGET .. Keys[key],
        configKey = Keys[key],
        help      = hi(loc("CONFIG_" .. key .. "_TT")),
        [DEPS]    = dep };
      end; -- textbox
  
  
  -- Functions available under RPTAGS.utils.config
  --
    Config.widget = {
      button   = button,
      check    = check,
      color    = color,
      h1       = h1,
      link     = link,
      list     = list,
      slider   = slider,
      tagedit  = tagedit,
      textbox  = textbox,
    };
  
  
  return "utils-widgets-trp3"
  
  end; -- if TRP3

end); -- startup function
  
  
